<?php

namespace App\Console\Commands;

use App\Http\Controllers\PaymentController;
use App\Http\Traits\EmailSender;
use App\Models\Beneficiary;
use App\Models\Benefit;
use App\Models\PaymentSchedule;
use App\Models\Payroll;
use App\Models\PayrollVerificationProcessLog;
use App\Models\StageEmail;
use Illuminate\Console\Command;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Crypt;

class GeneratePayroll extends Command
{
    use EmailSender;
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'generate:payroll';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'This command generated payrolls based on their scheduled dates. It is run in the scheduler everyday.';

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Execute the console command.
     *
     * @return int
     */
    public function handle()
    {
        $today = date('Y-m-d', strtotime(now()));
        $schedules = PaymentSchedule::whereDate('begin_period','<=',$today)->whereDate('end_period','>=',$today)->get();

        foreach ($schedules as $schedule) {
            $schedule_days = $schedule->day_of_payment;
            $number_of_occurrence = sizeof($schedule_days);

            if ($schedule->type == 'One-Off') $number_of_occurrence = 1;

            $payroll_exits = Payroll::where('payment_schedule_id','=',$schedule->id)
                ->where('benefit_id','=',$schedule->benefit_id)->count();
            if ($payroll_exits >= $number_of_occurrence) continue;

            foreach ($schedule_days as $schedule_day) {

                if ($schedule_day != date('d',strtotime(now()))) continue;

                $this->generateNewPayrolls($schedule->benefit_id,$schedule_day);

                $this->line($schedule->benefit->titles." payroll generated for ".$schedule_day." ".date('M, Y', strtotime(now())));

            }
        }

        return Command::SUCCESS;
    }
    public function generateNewPayrolls($benefit_id,$payment_day)
    {
        $benefit = Benefit::find($benefit_id);

        $beneficiaries = $this->getUpdatedBeneficiaries(cryptThis($benefit_id));

        try{
            $payroll = Payroll::updateOrCreate([
                'payment_date'          =>  date_create(date('Y-m-').$payment_day),
                'benefit_id'   =>  $benefit_id,
            ],[
                'payment_schedule_id'   =>  $benefit->paymentSchedule->id,
                'created_by'            =>  1,
                'beneficiaries_id'      => $beneficiaries
            ]);

            if ($payroll) {
                $this->movePayroll($payroll->id,'Payroll has been generated.');
            }
        }catch(\Exception $ex){
            return false;
        }
    }

    public function getUpdatedBeneficiaries($benefit_id)
    {
        $beneficiaries_list = array();
        $beneficiaries_number = array();

        if (Crypt::decrypt($benefit_id) == config('settings.christmas_bonus_id')) {
            $benefits = Benefit::where('main_benefit','=', 1)->where('archived','=',0)->get();
        }
        else {
            $benefits = Benefit::where('id','=',Crypt::decrypt($benefit_id))->get();
        }

        foreach ($benefits as $benefit) {
            $beneficiaries = Beneficiary::where('form_id','=',$benefit->form_id)->where('archived','=',0)->get();
            if((sizeof($beneficiaries)) < 1) continue;

            foreach ($beneficiaries as $beneficiary) {
                if (isEligibility($beneficiary,$benefit->id)) {
                    if (in_array($beneficiary->benefitRegistration->applicant_number,$beneficiaries_number)) {
                        continue;
                    }
                    else {
                        $beneficiaries_number[] = $beneficiary->benefitRegistration->applicant_number;
                    }
                    $beneficiaries_list[] = $beneficiary->id;
                }
            }
        }

        return $beneficiaries_list;
    }

    public function movePayroll($payroll_id,$message)
    {
        $payroll = Payroll::find($payroll_id);
        $submit_id = getStartProcessCode(config('settings.payment_process_lookupCode'));

        if (startVerification($submit_id)) {
            $beneficiaries = $this->getUpdatedBeneficiaries(cryptThis($payroll->benefit->id));
            $payroll->beneficiaries_id = $beneficiaries;
        }

        $payroll->stage_lookup_id = $submit_id;
        $payroll->status = 1;
        if ($payroll->save()) {

            $getRole = StageEmail::where('lookup_id','=',$submit_id)->pluck('roles')->first();
            $payroll->responsible_role = $getRole;
            $payroll->save();

            if ($payroll->save()) {
                PayrollVerificationProcessLog::create([
                    'payroll_id'                =>  $payroll->id,
                    'user_id'                   =>  1,
                    'verification_process_id'   =>  $submit_id,
                    'message'                   =>  $message
                ]);

                //Send Email Notification
                $this->send_email($submit_id,'payrolls',$payroll_id);
            }
        }
    }
}
