<?php

namespace App\Http\Controllers;

use App\Http\Traits\EmailSender;
use App\Mail\CaseReported;
use App\Models\Applicant;
use App\Models\Beneficiary;
use App\Models\CaseManagement;
use App\Models\CasesCommentsStage;
use App\Models\ContactDetail;
use App\Models\Deployment;
use App\Models\GeneralNotification;
use App\Models\Role;
use App\Models\User;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Crypt;
use Illuminate\Support\Facades\Http;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\URL;
use Illuminate\Support\Str;
use Yajra\DataTables\DataTables;

class CaseManagementController extends Controller
{
    use EmailSender;

    public function __construct()
    {
        $this->middleware(['auth']);
    }

    public function index(Request $request)
    {
        $cases = CaseManagement::with(['_category','_type','owner','beneficiary'])
            ->where('instance_id', getActiveInstance())->orderBy('created_at', 'asc');

        if (isset($request->category) && $request->category != 'Select'){
            $cases->where('category','=', $request->category);

        }

        if (isset($request->severity) && $request->severity != 'Select'){
            $cases->where('severity','=', $request->severity);
        }

        if (isset($request->year)){
            $cases->whereYear('created_at','=', $request->year);
        }

        if (isset($request->stage) && $request->stage != 'Select'){
            $cases->where('stage','=', $request->stage);
        }
        else {
            $cases->where('stage','!=', 'Close');
        }

        $total_cases = CaseManagement::where('instance_id','=',getActiveInstance())->count();
        $new_cases = CaseManagement::where('instance_id','=',getActiveInstance())->newCases()->count();
        $in_progress_cases = CaseManagement::where('instance_id','=',getActiveInstance())->inProgressCases()->count();
        $on_hold_cases = CaseManagement::where('instance_id','=',getActiveInstance())->onHoldCases()->count();
        $closed_cases = CaseManagement::where('instance_id','=',getActiveInstance())->closedCases()->count();

        $cases = $cases->paginate(100);

        return view('cases.list', compact('request','cases','total_cases','closed_cases','in_progress_cases','on_hold_cases','new_cases'));
    }

    public function myCases(Request $request)
    {
        $cases = CaseManagement::with(['_category','_type','owner','beneficiary'])
            ->where('applicant_number', '=',auth()->user()->applicant_number)
            ->orderBy('created_at', 'asc');

        if (isset($request->category) && $request->category != 'Select'){
            $cases->where('category','=', $request->category);

        }

        if (isset($request->severity) && $request->severity != 'Select'){
            $cases->where('severity','=', $request->severity);
        }

        if (isset($request->stage) && $request->stage != 'Select'){
            $cases->where('stage','=', $request->stage);
        }
        else {
            $cases->where('stage','!=', 'Close');
        }

        if ( isset($request->date_range)) {
            $dates = getDateTRange($request->date_range);
            $cases->where('created_at','>=',$dates['from'])->where('created_at','<=',$dates['to']);
        }

        $cases = $cases->paginate(100);
        return view('customers.my-cases', compact('request','cases'));
    }

    public function NewCase(Request $request)
    {
        $request->validate([
            'title'  =>  'required|min:3',
            'organization'  =>  'required|min:1',
            'category'  =>  'required|min:3',
//            'type'  =>  'required',
            'severity'  =>  'required|min:3',
            'description'  =>  'required|min:3',
            'beneficiary'  =>  'nullable',
            'file_upload'  =>  'nullable|min:3',
        ]);
        $deployment = Deployment::findOrfail($request->organization);

        $case = new CaseManagement();

        $alert_message = "A New Case has been created.";
        $type = "success";

        $case->stage = 'Opened';
        $case->applicant_number = auth()->user()->applicant_number;
        $case->user_id = Auth::id();
        $case->title = $request->title;
        $case->description = $request->description;
        $case->severity = $request->severity;
        $case->category = $request->category;
//        $case->type = $request->type;
        $case->name = auth()->user()->fullName;
        $case->contact = auth()->user()->phone??auth()->user()->email;
        $case->instance_id = $request->organization;


        if(isset($request->remove_file)) {
            $case->file_upload = null;
        }
        if($request->file('file_upload')) {
            $case->file_upload = $this->uploadDocument();
        }

        if ($case->save()) {
            Mail::to([$deployment->email])->send(new CaseReported($case));

            GeneralNotification::create([
                'user_id'   =>  auth()->id(),
                'receivers'   =>  [$deployment->admin->id],
                'subject_id'   =>  $case->id,
                'table'   =>  $case->getModel()->getTable(),
                'instance_id'   =>  getActiveInstance(),
                'message'   =>  '<strong>New case:</strong> '.content_excerpt($case->title,50).'<br><small>'.getSeverityLabel($case->severity).'</small>',
                'route'   =>  route('case.viewCase',[cryptThis($case->id)]),
            ]);


            session()->flash('status', $type);
            session()->flash('message', $alert_message);
        }
        else {
            session()->flash('status', 'error');
            session()->flash('message', 'Sorry! Something went wrong. Try again.');
        }

        return back();
    }

    public function viewMyCase($case_id)
    {

        $case = CaseManagement::find(Crypt::decrypt($case_id));

        if (!$case->canAccess()) return back();

        updateNotification($case->id,$case->getTable());

        $comments = $case->comments->sortBy('created_at');

        $isEditor = false;
        $canRespond = false;

        $stages = null;
        $current_stage = $case->stage;

        return view('customers.view-case',compact('case','comments',
            'stages','isEditor','current_stage','canRespond'));
    }

    /**
     * Get the list of Active Cases DataTable
     * @return mixed
     * @throws \Exception
     */
    public function getActiveCasesList(Request $request)
    {
        $cases = CaseManagement::where('archived','=',false)
            ->where('instance_id','=',getActiveInstance())->orderBy('updated_at','asc');
        if ( !is_null($request['amp;date_range'])) {
            $dates = getDateRange($request['amp;date_range']);
            $cases->where('created_at', '>=', $dates['from'])
                ->where('created_at', '<=', $dates['to']);
        }

        if (isset($request['severity']) && $request['severity'] != 'Select'){
            $cases->where('severity','=', $request['severity']);
        }

        if (isset($request['amp;stage']) && $request['amp;stage'] != 'Select'){
            $cases->where('stage','=', $request['amp;stage']);

        }
        $cases = $cases->get();

        return DataTables::of($cases)
            ->addColumn('action', function ($case) {
                if(Auth::id() == $case->user_id) {
                    $archive = '<a href="'.route('case.archiveCase',[cryptThis($case->id)]).
                        '" onclick="return confirm(\'Are you sure you want to archive this case?\')"
                         class="btn btn-light-warning btn-icon btn-sm mb-1 mr-1" title="'. __("Archive Case").'"><i class="la la-lock"></i></a>';
                    $edit = '<a href="'.route('case.editCase',[cryptThis($case->id)]).
                        '"  class="btn btn-light-primary btn-sm btn-icon mb-1 mr-1" title="'. __("Edit Case") .'"><i class="la la-edit"></i></a>';
                    $view = '<a href="'.route('case.viewCase',[cryptThis($case->id)]).'"
                         class="btn btn-light-success btn-sm btn-icon mb-1 mr-1" title="'. __("View Case") .'"><i class="la la-eye"></i></a>';

                    $delete = '<a href="'.route('case.deleteCase',[cryptThis($case->id)]).
                        '" onclick="return confirm(\'The case will be taken off the list completely . Do you want to proceed ? \')"
                         class="btn btn-light-danger btn-sm btn-icon mb-1 mr-1" title="'. __("Delete Case").'"><i class="la la-trash"></i></a>';

                    if($case->stage == 'Closed') {
                        $edit = null; $delete = null;
                    }

                    return $edit." ".$view." ".$archive." ".$delete;
                }
                else {
                    return '<a href="'.route('case.viewCase',[cryptThis($case->id)]).'" data-toggle="tooltip"
                        class="btn btn-sm btn-icon btn-light-success mb-1 mr-1" title="'. __("View Case") .'"><i class="la la-eye"></i></a>';
                }

            })
            ->editColumn('stage', function ($case) {
                return getCaseLabel($case->stage);
            })
            ->editColumn('title', function ($case) {
                return $case->title;
            })
            ->editColumn('severity', function ($case) {
                return getSeverityLabel($case->severity);
            })
            ->editColumn('created_at', function ($case) {
                return date('d M, Y',strtotime($case->created_at));
            })
            ->escapeColumns([])
            ->make();
    }

    /**
     * Archives
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View
     */
    public function archives()
    {
        return view('cases.archived-list');
    }

    /**
     * Get the list of Archived Cases DataTable
     * @return mixed
     * @throws \Exception
     */
    public function getArchivedCasesList(Request $request)
    {

        $cases = CaseManagement::where('archived','=',true)
            ->where('instance_id','=',getActiveInstance())->orderBy('updated_at','asc');

        if ( isset($request['amp;date_range'])) {
            $dates = getDateRange($request['amp;date_range']);
            $cases->where('created_at', '>=', $dates['from'])
                ->where('created_at', '<=', $dates['to']);
        }

        if (isset($request['severity']) && $request['severity'] != 'Select'){
            $cases->where('severity','=', $request['severity']);
        }

        if (isset($request['amp;stage']) && $request['amp;stage'] != 'Select'){
            $cases->where('stage','=', $request['amp;stage']);

        }
        $cases = $cases->get();
        return DataTables::of($cases)
            ->addColumn('action', function ($case) {
                if(Auth::id() == $case->user_id) {
                    $archive = '<a href="'.route('case.archiveCase',[cryptThis($case->id)]).
                        '" onclick="return confirm(\'Are you sure you want to archive this case?\')"
                         class="btn btn-light-warning btn-sm btn-icon mb-1 mr-1" title="'. __("Archive Case").'"><i class="la la-lock"></i></a>';
                    if ($case->archived) {
                        $archive = '<a href="'.route('case.archiveCase',[cryptThis($case->id)]).
                            '" onclick="return confirm(\'Are you sure you want to restore this case?\')"
                         class="btn btn-light-danger btn-sm btn-icon mb-1 mr-1" title="'. __("Restore Case").'"><i class="la la-lock-open"></i></a>';
                    }
                    $edit = '<a href="'.route('case.editCase',[cryptThis($case->id)]).
                        '"  class="btn btn-light-primary btn-sm btn-icon mb-1 mr-1" title="'. __("Edit Case") .'"><i class="la la-edit"></i></a>';
                    $view = '<a href="'.route('case.viewCase',[cryptThis($case->id)]).'"
                         class="btn btn-light-success btn-sm btn-icon mb-1 mr-1" title="'. __("View Case") .'"><i class="la la-eye"></i></a>';

                    $delete = '<a href="'.route('case.deleteCase',[cryptThis($case->id)]).
                        '" onclick="return confirm(\'The case will be taken off the list completely . Do you want to proceed ? \')"
                         class="btn btn-light-danger btn-sm btn-icon mb-1 mr-1" title="'. __("Delete Case").'"><i class="la la-trash"></i></a>';

                    if($case->stage == 'Close') {
                        $edit = null; $delete = null;
                    }
                    return $edit.' '.$view.' '.$archive.''.$delete;
                }
                else {
                    return '<a href="'.route('case.viewCase',[cryptThis($case->id)]).'"
                        class="btn btn-sm btn-icon btn-light-success mr-1 mb-1" title="'. __("View Case") .'"><i class="la la-eye"></i></a>
                        <a href="'.route('case.archiveCase',[cryptThis($case->id)]).
                        '" onclick="return confirm(\'Are you sure you want to restore this case?\')"
                         class="btn btn-success btn-sm btn-icon mb-1 mr-1" title="'. __("Restore Case").'"><i class="la la-lock-open"></i></a>';
                }

            })
            ->editColumn('stage', function ($case) {
                return getCaseLabel($case->stage);
            })
            ->editColumn('title', function ($case) {
                return $case->title;
            })
            ->editColumn('severity', function ($case) {
                return getSeverityLabel($case->severity);
            })
            ->editColumn('created_at', function ($case) {
                return date('d M, Y',strtotime($case->created_at));
            })
            ->escapeColumns([])
            ->make();
    }

    /**
     * Create Case
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View
     * @throws \Throwable
     */
    public function createCase()
    {
        $memberships = Deployment::with(['applicants'])->where('id','=',getActiveInstance())->get();
        $beneficiaries = Applicant::orderBy('surnames')->get();
        $page_title = __('Create Case');
        $action = __('create');
        $case_id = null;
        return view('cases.form', compact('memberships','page_title','case_id','action', 'beneficiaries'));
    }

    /**
     * Edit Case
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View
     * @throws \Throwable
     */
    public function editCase($case_id)
    {
        $case = CaseManagement::with(['_category','_type','owner','beneficiary'])
            ->where('instance_id','=',getActiveInstance())
            ->where('id','=',Crypt::decrypt($case_id))->first();
        if(!$case) {
            $message = "Invalid Case";
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back()->withInput();
        }
        $beneficiary = $case->beneficiary?->regID;
        $page_title = __('Edit Case');
        $action = __('edit');
        return view('cases.form', compact('beneficiary','page_title','case_id','action','case'));
    }
    public function uploadDocument($key='file_upload',$location='cases', $validation = 'file|mimes:jpeg,png,jpg,gif,docx,pdf,txt,xlsx|max:10048|sometimes')
    {
        $instance = getActiveInstance();
        request()->validate([$key => $validation]);

        $file = null;
        if (request()->hasFile($key)) {
            $file = request()->file($key);
            $name = $file->hashName();
            $file = Storage::disk('local')->putFileAs($instance.'/'.$location, request()->file($key),$name);
        }
        return $file;
    }

    /**
     * Make corresponding changes to API
     * Save Case information
     * @param Request $request
     * @return \Illuminate\Http\RedirectResponse
     */
    public function saveCase(Request $request)
    {
        //dd(substr($request->contact, 1, 12));
        $request->validate([
            'name'  =>  'required|min:3',
            'contact'  =>  'required|min:3|regex:/^\+\d{1,3}\d{3,14}$/',
            'reference'  =>  'nullable|min:3',
            'title'  =>  'required|min:3',
            'category'  =>  'required|min:3',
//            'type'  =>  'required',
            'severity'  =>  'required|min:3',
            'description'  =>  'required|min:3',
            'beneficiary'  =>  'nullable',
            'file_upload'  =>  'nullable|min:3',
        ]);

        $owner = null;
        if (!is_null($request->beneficiary)) {
            $beneficiary = Applicant::where('regID', '=', $request->beneficiary)->first();
            if(!$beneficiary) {
                $message = "Invalid Beneficiary ID provided";
                session()->flash('status', 'error');
                session()->flash('message', $message);
                return back()->withInput();
            }
            if ($beneficiary->owner) {
                $owner = $beneficiary->owner->id;
            }
        }
        if ($request->case_id == null) {
            $case = new CaseManagement();
            $is_new = true;

            $alert_message = "A New Case has been created.";
            $type = "success";
            $case->stage = 'Opened';
        }
        else {
            $case = CaseManagement::find(Crypt::decrypt($request->case_id));
            $alert_message = "The Case has been updated.";
            $type = "success";
            $is_new = false;
        }


        switch($request->category){
            case 179:
                $cat_code = 'FIN';
                break;
            case 180:
                $cat_code = 'THT';
                break;
            case 210:
                $cat_code = 'ACC';
                break;
            case 211:
                $cat_code = 'NAT';
                break;
            case 212:
                $cat_code = 'CAL';
                break;
            default:
                $cat_code = 'CAS';
                break;
            
        }

        switch($request->category){
            case 179:
                $cat_code = 'FIN';
                break;
            case 180:
                $cat_code = 'THT';
                break;
            case 210:
                $cat_code = 'ACC';
                break;
            case 211:
                $cat_code = 'NAT';
                break;
            case 212:
                $cat_code = 'CAL';
                break;
            default:
                $cat_code = 'CAS';
                break;
            
        }

        switch($request->severity){
            case 'Low':
                $sev_code = '01';
                break;
            case 'Normal':
                $sev_code = '02';
                break;
            case 'Critical':
                $sev_code = '03';
                break;
            
            default:
                $cat_code = 'CAS';
                break;
            
        }

        $case->applicant_number = $request->beneficiary;
        $case->user_id = Auth::id();
        $case->title = $request->title;
        $case->description = $request->description;
        $case->severity = $request->severity;
        $case->category = $request->category;
//        $case->type = $request->type;
        $case->name = Str::headline($request->name);
        $case->contact = $request->contact;
        $case->reference = $request->reference;
        $case->instance_id = getActiveInstance();

        if(isset($request->remove_file)) {
            $case->file_upload = null;
        }
        if($request->file('file_upload')) {
            $case->file_upload = $this->uploadDocument();
        }

        if ($case->save()) {
            if ($is_new) {
                $deployment = getActiveDeployment();
                if (!is_null($owner)) {
                    $receivers = [$owner,$deployment->admin->id];
                }
                else {
                    if($deployment && $deployment->admin){

                        $receivers = [$deployment->admin->id];
                    }else{
                        $receivers = [];
                    }
                    
                }
                
                $case->reference = $cat_code.'-'.$sev_code.'-'.$case->id;
                $case->save();
                //Send SMS Notification to beneficiary

                try {
                    $apiKey = env('ARKESEL_API_KEY');
                    $message = 'A case has been opened on your behalf. Case Ref : '.$case->reference.'. Case Title : '.$case->title;
                    $response = Http::get("https://sms.arkesel.com/sms/api", [
                        'action' => 'send-sms',
                        'api_key' => $apiKey,
                        'to' => substr($case->contact, 1, 12),
                        'from' => 'Tendledger',
                        'sms' => $message,
                    ]);
                } catch (Exception $e) {
                    session()->flash('status', 'error');
                    session()->flash('message', 'New case created but SMS notification failed to send');
                }

                //
                GeneralNotification::create([
                    'user_id'   =>  auth()->id(),
                    'receivers'   =>  $receivers,
                    'subject_id'   =>  $case->id,
                    'table'   =>  $case->getModel()->getTable(),
                    'instance_id'   =>  getActiveInstance(),
                    'message'   =>  '<strong>New case:</strong> '.content_excerpt($case->title,50).'<br><small>'.getSeverityLabel($case->severity).'</small>',
                    'route'   =>  route('case.viewCase',[cryptThis($case->id)]),
                ]);
            }
            session()->flash('status', $type);
            session()->flash('message', $alert_message);
        }
        else {
            session()->flash('status', 'error');
            session()->flash('message', 'Sorry! Something went wrong. Try again.');
        }

        return redirect()->route('case.list');
    }

    public function viewCase($case_id)
    {
        $page_title = __('View Case');

        $roles = Role::all();
        $users = User::where('email_verified_at','!=', null)->where('deleted_at','=', null)->get();

        $case = CaseManagement::with(['comments'])->where('id','=', Crypt::decrypt($case_id))->first();

        updateNotification($case->id,$case->getTable());

        $comments = $case->comments->sortBy('created_at');

        $get_users_roles = [];
        if (!is_null($case->user_ids)) {
            $get_users_roles = getUsersEmailAndNameByUsers($case->user_ids);
        }
        elseif (!is_null($case->role_ids)) {
            $get_users_roles = getRolesByID($case->role_ids);
        }
        $from_archives = false;
        if(app('router')->getRoutes()->match(app('request')->create(URL::previous()))->getName() == 'case.archives') {
            $from_archives = true;
        }

        $isEditor = false;
        $canRespond = false;
        if (!is_null($case->role_ids)) {
            $user = isUserEligibleByRole($case->role_ids,auth()->id());
            if ($user->count() > 0) {
                $isEditor = true;
                $canRespond = true;
            }
        }
        elseif (!is_null($case->user_ids)) {
            if (in_array(Auth::id(),$case->user_ids)) {
                $isEditor = true;
                $canRespond = true;
            }
        }
        if ($case->user_id == Auth::id() || Auth::user()->hasRole('administrator')) {
            $canRespond = true;
        }

        $stages = null;
        $current_stage = $case->stage;
        if (Auth::user()->hasRole('administrator')) {
            $stages = config('settings.case_stage');
        }
        elseif ($isEditor) {
            $stages = config('settings.case_stage_for_assigned');
        }
        elseif (Auth::id() == $case->user_id) {
            if($current_stage == "Resolved") {
                $current_stage = 'Close';
            }
            $stages = config('settings.case_stage_for_owner');
        }

        return view('cases.view',compact('page_title','case','comments','from_archives',
            'stages','roles','users','isEditor','current_stage','canRespond','get_users_roles'));
    }

    /**
     * Archive/Restore a Case
     * @param $case_id
     * @return \Illuminate\Http\RedirectResponse
     */
    public function archiveCase($case_id)
    {
        $case = CaseManagement::find(Crypt::decrypt($case_id));

        if ($case->archived == 1) {
            $case->archived = false;
            $case->save();
            session()->flash('status', 'success');
            session()->flash('message', 'Case has been restored.');
        }
        else {
            $case->archived = true;
            $case->save();
            session()->flash('status', 'success');
            session()->flash('message', 'Case has been archived.');
        }

        return back();
    }

    /**
     * @param Request $request
     * @param $case_id
     *
     * @return \Illuminate\Http\RedirectResponse
     * @throws \Illuminate\Validation\ValidationException
     */
    public function assignStage(Request $request,$case_id)
    {
        $this->validate($request, [
            'assign_to' => 'required|string|min:4',
        ]);

        $case = CaseManagement::find(Crypt::decrypt($case_id));
        $case->stage = "Pending Review";

        $message = "Dear All <br><br>";
        if ($request->assign_to == 'roles') {
            $case->role_ids = $request->role_ids;
            $case->user_ids = null;
            $emails = getUsersEmailAndNameByRoles($request->role_ids);
        }
        else {
            $case->role_ids = null;
            $case->user_ids = $request->user_ids;
            $emails = getUsersEmailAndNameByUsers($request->user_ids);
            if (sizeof($request->user_ids) == 1) {
                $message = "Dear Sir/Madam <br><br>";
            }
        }

        if ($case->save()) {
            $message .= __("A case has been opened for your support. The details are below.");
            $message .= "<br><br>";
            $message .= "Subject: <br><strong>".$case->title."</strong><br><br>";
            $message .= "Description: <br>";
            $message .= $case->description;
            $message .= "<br><br> Thank you.<br><br>Best Regards,";
            $subject = "A New Complaint has been assigned";

            $this->blast_mails($message,$subject,$emails);

            $message = __("The Complaint has been assigned.");
            $type = "success";
            alertNotify($message,null,$type);
            return redirect()->route('case.list');
        }
        else {
            $message = __("Sorry! Something went wrong. Please try again.");
            $type = "danger";
            alertNotify($message,null,$type);
            return back();
        }
    }

    public function saveResponse(Request $request, $case_id)
    {
        $request->validate([
            'response'  =>  'required|string|min:10'
        ]);
        $case = CaseManagement::find(Crypt::decrypt($case_id));

        if (isset($request->stage)) {
            $case->stage = $request->stage;
            $case->save();
        }

        if(isset($request->remove_file)) {
            $case->file_upload = null;
        }

        $comment = new CasesCommentsStage();
        $comment->case_id = Crypt::decrypt($case_id);
        $comment->responder = Auth::id();
        if (isset($request->stage)) {$comment->stage = $request->stage;}
        else {$comment->stage = $case->stage;}
        $comment->comment = $request->response;
        if($request->file('file_input')) {
            $file = $this->uploadDocument('file_input',$case->deployment_id);
            $comment->attachment = $file;
        }

        if ($comment->save()) {
            $deployment = Deployment::find($case->instance_id);
            if (auth()->user()->hasRole('beneficiary')) {
                GeneralNotification::create([
                    'user_id'   =>  auth()->id(),
                    'receivers'   =>  [$deployment->admin->id],
                    'subject_id'   =>  $case->id,
                    'table'   =>  $case->getModel()->getTable(),
                    'instance_id'   =>  $case->instance_id,
                    'message'   =>  '<strong>Case #'.$case->id.' commented:</strong> '.content_excerpt($comment->comment,50),
                    'route'   =>  route('case.viewCase',[cryptThis($case->id)]),
                ]);
            }
            elseif (!auth()->user()->hasRole('beneficiary')) {
                if (isset($case->beneficiary->owner)) {
                    GeneralNotification::create([
                        'user_id'   =>  auth()->id(),
                        'receivers'   =>  [$case->beneficiary->owner->id],
                        'subject_id'   =>  $case->id,
                        'table'   =>  $case->getModel()->getTable(),
                        'instance_id'   =>  $case->instance_id,
                        'message'   =>  '<strong>Case #'.$case->id.' commented:</strong> '.content_excerpt($comment->comment,50),
                        'route'   =>  route('case.viewMyCase',[cryptThis($case->id)]),
                    ]);
                }
            }
            if (auth()->id() != $case->user_id && !$case->owner->hasRole('beneficiary')) {
                GeneralNotification::create([
                    'user_id'   =>  auth()->id(),
                    'receivers'   =>  [$case->user_id],
                    'subject_id'   =>  $case->id,
                    'table'   =>  $case->getModel()->getTable(),
                    'instance_id'   =>  $case->instance_id,
                    'message'   =>  '<strong>Case #'.$case->id.' commented:</strong> '.content_excerpt($comment->comment,50),
                    'route'   =>  route('case.viewCase',[cryptThis($case->id)]),
                ]);
            }

//            if($request->stage == 'Resolved') {
//                $message = "Dear Sir/Madam <br><br>";
//                $message .= __("The Case with Subject,");
//                $message .= " <strong>".$case->title."</strong>, has been marked as RESOLVED. Please review and CLOSE the case if issue has been addressed.";
//                $message .= "<br><br> Thank you.<br><br>Best Regards,";
//                $subject = "A Case has been Resolved";
//
//                $cc = getUsersEmailAndNameByRoles([config('settings.admin_role_id')]);
//                $emails = getUsersEmailAndNameByUsers([Auth::id()]);
//                $this->blast_mails($message,$subject,$emails,$cc);
//            }
//            if($request->stage == 'Pending Review' && Auth::id() != $case->user_id) {
//                $message = "Dear Sir/Madam <br><br>";
//                $message .= __("A new comment has been added to your Case with Subject,");
//                $message .= " <strong>".$case->title."</strong>.";
//                $message .= " <br><br> Comment: <br>".$request->response;
//                $message .= "<br><br> Thank you.<br><br>Best Regards,";
//                $subject = "New Case Comment";
//
//                $emails = getUsersEmailAndNameByUsers([Auth::id()]);
//                $this->blast_mails($message,$subject,$emails);
//            }
            $message = "The Comment has been added.";
            $type = "success";
        }
        else {
            $message = "Sorry! Comment wasn't saved. Please try again.";
            $type = "danger";
        }
        session()->flash('status', $type);
        session()->flash('message', $message);
        return back();
    }

    public function updateCategory(Request $request, $case_id)
    {
        $case = CaseManagement::findOrFail(Crypt::decrypt($case_id));
        if (!in_array($request->updateCategory,array_keys(getCaseCategories()->toArray()))){
            session()->flash('status', 'error');
            session()->flash('message', 'Invalid case category selected.');
            return back();
        }

        $all_categories = getCaseCategories()->toArray();
        $previous_category = $all_categories[$case->category];
        $new_category = $all_categories[$request->updateCategory];

        $comment = "Case category has been changed from $previous_category to ".$new_category;
        $case->category = $request->updateCategory;
        if ($case->save()) {
            $this->addResponse($case,$comment);
            $message = 'Case category has been changed successfully.';
            $type = 'success';
        }
        else {
            $message = 'Sorry! Kindly try again.';
            $type = 'error';
        }
        session()->flash('status', $type);
        session()->flash('message', $message);
        return back();
    }

    public function updateSeverity(Request $request, $case_id)
    {
        $case = CaseManagement::findOrFail(Crypt::decrypt($case_id));
        if (!in_array($request->updateSeverity,config('settings.severity'))){
            session()->flash('status', 'error');
            session()->flash('message', 'Invalid case severity selected.');
            return back();
        }
        $previous_severity = $case->severity;
        $comment = "Case severity has been changed from $previous_severity to ".$request->updateSeverity;
        $case->severity = $request->updateSeverity;
        if ($case->save()) {
            $this->addResponse($case,$comment);
            $message = 'Case severity has been changed successfully.';
            $type = 'success';
        }
        else {
            $message = 'Sorry! Kindly try again.';
            $type = 'error';
        }
        session()->flash('status', $type);
        session()->flash('message', $message);
        return back();
    }

    public function escalateCase($case_id)
    {
        $case = CaseManagement::findOrFail(Crypt::decrypt($case_id));
        if ($case->severity == 'Critical'){
            session()->flash('status', 'info');
            session()->flash('message', 'This case is already at its highest level.');
            return back();
        }

        if ($case->severity == 'Normal') {
            $new_level = 'Critical';
        }
        else {
            $new_level = 'Normal';
        }

        $comment = "Case has been escalated to $new_level Impact";
        $case->severity = $new_level;

        if ($case->save()) {
            $this->addResponse($case,$comment);
            $message = 'Case has been escalated successfully.';
            $type = 'success';
        }
        else {
            $message = 'Sorry! Kindly try again.';
            $type = 'error';
        }
        session()->flash('status', $type);
        session()->flash('message', $message);
        return back();
    }

    public function updateType(Request $request, $case_id)
    {
        $case = CaseManagement::findOrFail(Crypt::decrypt($case_id));
        if (!in_array($request->updateType,array_keys(getCaseTypes()->toArray()))){
            session()->flash('status', 'error');
            session()->flash('message', 'Invalid case type selected.');
            return back();
        }
        $all_types = getCaseTypes()->toArray();
        $new_type = $all_types[$request->updateType];

        $previous_type = $case->_type->fullName;
        $comment = "Case type has been changed from $previous_type to ".$new_type;
        $case->type = $request->updateType;
        if ($case->save()) {
            $this->addResponse($case,$comment);
            $message = 'Case type has been changed successfully.';
            $type = 'success';
        }
        else {
            $message = 'Sorry! Kindly try again.';
            $type = 'error';
        }
        session()->flash('status', $type);
        session()->flash('message', $message);
        return back();
    }

    public function addResponse(CaseManagement $case,$message,$stage=null)
    {
        $comment = new CasesCommentsStage();
        $comment->case_id = $case->id;
        $comment->responder = Auth::id();
        if (isset($stage)) {
            $comment->stage = $stage;
        }
        else {
            $comment->stage = $case->stage;
        }
        $comment->comment = $message;
        $comment->save();

        return $comment;
    }

    /**
     * Delete a Case
     * @param $case_id
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View
     * @throws \Throwable
     */
    public function deleteCase($case_id)
    {
        if (CaseManagement::destroy(Crypt::decrypt($case_id))) {
            $message = "Case has been deleted";
            session()->flash('status', 'success');
            session()->flash('message', $message);
        }
        else {
            $message = 'Sorry! Something went wrong. Try again.';
            session()->flash('status', 'error');
            session()->flash('message', $message);
        }
        return back();
    }
}
