<?php

namespace App\Http\Controllers;

use App\Http\Middleware\LoadUserPermissions;
use App\Models\Country;
use App\Models\Lookup;
use App\Models\LookupCode;
use App\Models\Office;
use App\Models\Permission;
use App\Models\Role;
use App\Models\User;
use Illuminate\Database\QueryException;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Crypt;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Str;
use Laratrust\Models\LaratrustRole;

class OfficesController extends Controller
{
    public function __construct()
    {
        $this->middleware(['auth:web', LoadUserPermissions::class]);
    }

    public function index(Request $request){

        //$status = Status::query()->where('status','Active')->first();
        $statues_lookup_code = LookupCode::query()->where('lookUpCode', 'STATUS')->first()?->id;
        $status = Lookup::where("lookup_code_id", $statues_lookup_code)->first();
        $users = User::with('roles')->paginate(20);
        $roles = Role::orderBy('display_name')->where('instance_id','=', getActiveInstance())
            ->orWhere('is_general','=',1)->get();
//        $offices = Office::query()->where('instance_id','=', getActiveInstance())->get();
        $office_classes = Lookup::where('lookup_code_id', 23)->orderBy('order', 'asc')->get();

        if(empty($request->search_query)){
            $offices = Office::query()->where('instance_id','=', getActiveInstance())->paginate(20);
        }else{
            $offices = office::where('name', 'like', '%' . $request->search_query . '%')
                ->where('type', $request->type)
                ->where('instance_id','=', getActiveInstance())
                ->paginate(20);
        }

        $countries = Country::query()->orderBy('country_name','ASC')->get();

        return view( 'offices.index', compact('users', 'roles','offices','office_classes', 'countries' ) );
    }

    public function emptyForm(){
        $types = getOfficeTypes();
        $locations = getLocations();
        $leads = Role::where('instance_id','=', getActiveInstance())->orWhere('is_general','=',1)->get();
        $users = User::where('instance_id','=', getActiveInstance())->get();
        $theView = view('offices.empty_form', compact('types','locations','leads','users'));
        return $theView;
    }

    public function create(Request $request){
        $this->validate($request,[
            'name' => 'required|string|min:2|unique:offices,name',
            'location' => 'required|integer',
            'type' => 'required|integer',
        ]);

        try{
            $office = new Office();
            $office->name   = $request->name;
            $office->instance_id   = getActiveInstance();
            $office->type = $request->type;
            $office->parent_id = $request->parent_office;
            $office->location  = $request->location;
            $office->coverable_date  = date('Y-m-d', strtotime($request->coverable_date));
            $office->status  = 1;
            $office->code  = $request->code;
            $office->save();

            $message = 'A New Office has been added!';
            session()->flash('message', $message);
            session()->flash('status', 'success');

        }catch(\Illuminate\Database\QueryException $ex){
            $message = 'Office may already exist or check enter appropriate data! '.$ex->getMessage();
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back();
        }

        return back();
    }

    public function edit(Request $request){
        $office = Office::find($request->office_id);
        $types = getOfficeTypes();
        $locations = getLocations();
        $leads = Role::where('instance_id','=', getActiveInstance())->orWhere('is_general','=',1)->get();
        $users = User::where('instance_id','=', getActiveInstance())->get();

        $theView = view('offices.edit', compact('office','leads','types','locations','users'))->render();
        return response()->json(['theView'=>$theView]);
    }

    public function update(Request $request)
    {
        $this->validate($request,[
            'name' => 'required|string|min:2',
//            'lead' => 'nullable|integer',
//            'manager' => 'required|integer',
            'location' => 'required|integer',
            'type' => 'required|integer',
            'office_id' => 'required|integer',
//            'code' => 'required|string',
        ]);

        try {
            $office = Office::find($request->office_id);
            $office->name   = $request->name;
            $office->code    = $request->code;
            $office->lead    = $request->lead;
//            $office->manager = $request->manager;
            $office->type = $request->type;
            $office->parent_id = $request->parent_office;
            $office->location  = $request->location;
            $office->save();

            $message = 'Office information has been updated!';
            session()->flash('message', $message);
            session()->flash('status', 'success');

            return back();
        }catch (QueryException $exception){
            $message = 'The Office name may already exist or check enter appropriate data!'.$exception->getMessage();
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back();
        }
    }

    public function remove($id)
    {
        try {
            $office = Crypt::decrypt($id);
            Office::destroy($office);

            $message = 'Office has been removed!';
            session()->flash('message', $message);
            session()->flash('status', 'success');
            return back();
        }catch (QueryException $exception){
            $message = 'Something went wrong. Try again or contact Administrator.';
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back();
        }
    }

    public function activation($id)
    {
        try {
            $office = Office::find(Crypt::decrypt($id));
            if (!is_null($office)) {
                if ($office->status == 1){
                    $office->status = 0;
                    $message = 'The Office has been deactivated!';
                }
                else {
                    $office->status = 1;
                    $message = 'The Office has been activated!';
                }
                $office->save();

                session()->flash('message', $message);
                session()->flash('status', 'success');
            }
            else {
                $message = 'Office does not exist.';
                session()->flash('message', $message);
                session()->flash('status', 'error');
            }

            return back();
        }catch (QueryException $exception){
            $message = 'Something went wrong. Try again or contact Administrator.';
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back();
        }
    }

    public function hierarchy(){
        $statues_lookup_code = LookupCode::where('lookUpCode', 'STATUS')->first()?->id;
        $status = Lookup::where("lookup_code_id", $statues_lookup_code)->first();

        $offices = Office::with('children')->where('instance_id','=',getActiveInstance())
            ->whereNull('parent_id')->get();
        $countries = Country::all();

        $all_offices = Office::where('instance_id','=',getActiveInstance())->get();
        $office_classes = Lookup::where('lookup_code_id', 23)->orderBy('order', 'asc')->get();
        return view('offices.hierarchy', compact('offices', 'countries', 'all_offices', 'office_classes'));
    }

    public function deactivateOffice(Request $request){

        $office = Office::find($request->office_id);
        $message = 'The Office has been deactivated!';
        $office->status = 0;
        $office->save();

        session()->flash('message', $message);
        session()->flash('status', 'success');

        return redirect()->route('offices.list');
    }

    public function activateOffice(Request $request){

        $office = Office::find($request->office_id);
        $message = 'The Office has been activated!';
        $office->status = 1;
        $office->save();

        session()->flash('message', $message);
        session()->flash('status', 'success');

        return redirect()->route('offices.list');
    }
}
