<?php

namespace App\Http\Controllers;

use App\Models\Partner;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Crypt;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Storage;
use Illuminate\Validation\Rule;

class PartnersController extends Controller
{
    /**
     * Get Partners Data
     * @param Request $request
     *
     * @return string
     */
    public function index(Request $request)
    {
        $page_title = "Partners";

        $partners = Partner::with(['_type'])->where('instance_id','=',getActiveInstance());

        if (isset($request->name_query)) {
            $partners = $partners->where('name','like','%'.$request->name_query.'%');
        }
        if (isset($request->status_query)) {
            $partners = $partners->where('status','=',$request->status_query);
        }
        if (isset($request->type)) {
            $partners = $partners->where('type','=',$request->type);
        }
        $partners = $partners->get();

        return view('partners.index', compact('page_title','partners'));
    }

    /**
     * Create Partner
     *
     * @return string
     */
    public function create()
    {
        $action = 'new';
        $page_title = 'New Partner';
        return view('partners.form', compact('page_title','action'));
    }

    public function uploadLogo($key = 'logo', $validation = 'image|mimes:jpeg,png,jpg,gif,svg|max:2048|sometimes')
    {
        request()->validate([$key => $validation]);

        $file = null;
        if (request()->hasFile($key)) {
            $location = 'programs';
            $file = request()->file($key);
            $name = $file->hashName();
            $file = Storage::disk('public')->putFileAs($location, request()->file($key),$name);
            $file = 'storage/'.$file;
        }

        return $file;
    }

    /**
     * Save Partner
     *
     * @return string
     */
    public function save(Request $request)
    {
        $request->validate([
            'name'  =>  'required|string|unique:partners,name',
            'phone'  =>  'required|string',
            'email'  =>  'required|email',
            'type'  =>  'required',
            'website'  =>  'nullable|string',
            'status'  =>  'required|max:1',
            'description'  =>  'required|string',
            'logo'  =>  'nullable|file',
        ]);

        $document_path = null;
        if($request->file('logo')) {
            $document_path = $this->uploadLogo();
        }

        $partner = Partner::create([
            'name'  =>  $request->name,
            'instance_id'  =>  getActiveInstance(),
            'phone'  =>  $request->phone,
            'email'  =>  $request->email,
            'type'  =>  $request->type,
            'website'  =>  $request->website,
            'description'  =>  $request->description,
            'logo_url'  =>  $document_path,
            'status'  =>  $request->status,
            'created_by'  =>  auth()->id(),
        ]);

        if($partner) {
            $message = "A new Partner has been added.";
            session()->flash('status', 'success');
            session()->flash('message', $message);
            return back();
        }
        else {
            $message = "Sorry! Something went wrong. Please try again.";
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back()->withInput();
        }
    }

    /**
     * Edit Partner
     *
     * @return string
     */
    public function edit($partner_id)
    {
        $action = 'edit';
        $page_title = 'Edit Partner';
        $partner = Partner::findorFail(Crypt::decrypt($partner_id));
        return view('partners.form', compact('page_title','action','partner'));
    }

    /**
     * Update Partner
     *
     * @return string
     */
    public function update(Request $request)
    {
        $id = Crypt::decrypt($request->partner_id);
        $request->validate([
            'name'  =>  [
                'required',
                Rule::unique('partners')->ignore($id,'id'),
            ],
            'phone'  =>  'required|string',
            'email'  =>  'required|email',
            'type'  =>  'required',
            'website'  =>  'nullable|string',
            'status'  =>  'required|max:1',
            'description'  =>  'required|string',
            'logo'  =>  'nullable|file',
        ]);

        $partner = Partner::findOrFail($id);

        if(!$partner) {
            $message = "Sorry! Record does not exist in our system.";
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back()->withInput();
        }
        $partner->name = $request->name;
        $partner->phone = $request->phone;
        $partner->email = $request->email;
        $partner->type = $request->type;
        $partner->website = $request->website;
        $partner->description = $request->description;
        $partner->status = $request->status;
        $partner->updated_by = auth()->id();

        if($request->file('logo')) {
            $partner->logo_url = $this->uploadLogo();
        }

        if($partner->save()) {
            $message = "Partner has been updated.";
            session()->flash('status', 'success');
            session()->flash('message', $message);
            return redirect()->route('partner.list');
        }
        else {
            $message = "Sorry! Something went wrong. Please try again.";
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back()->withInput();
        }
    }

    /**
     * View Partner
     *
     * @return string
     */
    public function view($partner_id)
    {
        $id = Crypt::decrypt($partner_id);
        $partner = Partner::with([
            'programs',
            'updatedBy:id,name,firstname,othername',
            'createdBy:id,name,firstname,othername'])
        ->where('id','=',$id)->first();
        if(!$partner) {
            $message = "Sorry! Record does not exist in our system.";
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back()->withInput();
        }
        $page_title = 'View Partner';
        return view('partners.view', compact('page_title','partner'));
    }

    /**
     * Delete Partner
     *
     * @return string
     */
    public function delete($partner_id)
    {
        Partner::findorFail(Crypt::decrypt($partner_id));

        return back();
    }
}
