<?php

namespace App\Http\Controllers;

use App\Http\Traits\EmailSender;
use App\Http\Traits\FormSubmission;
use App\Http\Traits\VerificationProcesses;
use App\Models\Application;
use App\Models\BeneficiariesFormMapping;
use App\Models\Beneficiary;
use App\Models\Benefit;
use App\Models\DebtRecovery;
use App\Models\DebtRecoveryAmountPaidHistory;
use App\Models\Field;
use App\Models\Form;
use App\Models\LookupCode;
use App\Models\PaymentSchedule;
use App\Models\Payroll;
use App\Models\PayrollHistory;
use App\Models\PayrollVerificationProcessLog;
use App\Models\PayrollViewLog;
use App\Models\StageEmail;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Crypt;
use Illuminate\Support\Facades\Storage;
use PhpOffice\PhpSpreadsheet\Spreadsheet;
use PhpOffice\PhpSpreadsheet\Style\Alignment;
use Yajra\DataTables\DataTables;

class PaymentControllerOld extends Controller
{
    use FormSubmission;
    use EmailSender;
    use VerificationProcesses;

    public function __construct()
    {
        $this->middleware(['auth', 'verified']);
    }

    /********************
     * PAYMENT SCHEDULE
     ********************/

    public function paymentSchedule()
    {
        $benefits = Benefit::where('archived','=',false)->orderBy('titles','asc')->get();
        $schedules = PaymentSchedule::all();

        return view('payment.index',compact('benefits','schedules'));
    }

    public function createPaymentSchedule(Request $request)
    {
        $benefit = Benefit::find($request->benefit);
        if(!$benefit) {
            alertNotify(__('Sorry! Please select an existing Benefit.'), null, 'info');
            return back();
        }
        if(!isset($benefit->form)) {
            alertNotify(__('Sorry! this Benefit has not been assigned a form.'), null, 'info');
            return back();
        }
        $schedule = $benefit->paymentSchedule;

        return view('payment.create-payment-schedule',compact('benefit','schedule'));
    }

    public function savePaymentSchedule(Request $request)
    {
        $this->validate($request,[
            'type' => 'required|string|min:2',
            'day_of_payment' => 'required|string|min:1',
            'benefit' => 'required|numeric|min:1',
            'pension_amount' => 'nullable|array|min:1'
        ]);

        try{
            $owner = PaymentSchedule::updateOrCreate([
                'benefit_id'   =>  $request->benefit
            ],[
                'type'              =>  $request->type,
                'day_of_payment'    =>  $request->day_of_payment,
                'amount'            =>  $request->amount,
                'tax'            =>  $request->tax,
                'pension_amount'    =>  $request->pension_amount,
                'funeral_amount'    =>  $request->funeral_amount,
                'child_age_limit'    =>  $request->child_age_limit,
                'user_id'           =>  Auth::id(),
            ]);

            if ($owner) {
                alertNotify(__('Payment Schedule has been saved!'), null, 'success');
            }
            else {
                alertNotify(__('Sorry! Something went wrong. Try again'), null, 'danger');
            }

        }catch(\Exception $ex){
            alertNotify(__($ex->getMessage()), null, 'warning');
            return back();
        }

        return redirect()->route('payment.paymentSchedule');
    }

    public function editPaymentSchedule(Request $request)
    {
        $schedules = PaymentSchedule::find($request->id);

        return response()->json($schedules);
    }

    public function deletePaymentSchedule($id)
    {
        try {
            $schedule_id = Crypt::decrypt($id);

            if (PaymentSchedule::destroy($schedule_id)) {
                alertNotify(__('Payment Schedule has been deleted!'), null, 'success');
            }
            else {
                alertNotify(__('Sorry! Something went wrong. Please try again.'), null, 'danger');
            }
        }catch (\Exception $exception){
            alertNotify(__($exception->getMessage()), null, 'warning');
        }
        return back();
    }


    /***************
     * PAYROLL
     ***************/

    public function payroll(Request $request)
    {
        $benefits = Benefit::where('archived','=',0)->orderBy('titles','asc')->get();
        $year = date('Y', strtotime(now()));

        if (isset($request->year)) {
            $year = $request->year;
        }

        //Get selected year payroll
        $payrolls = Payroll::whereYear('created_at','=',$year)->orderBy('created_at','asc')->get();

//        $today = date('Y-m-d', strtotime(now()));
//        $schedules = PaymentSchedule::whereDate('begin_period','<=',$today)->whereDate('end_period','>=',$today)->get();
//
//        foreach ($schedules as $schedule) {
//            $schedule_days = $schedule->day_of_payment;
//            $number_of_occurrence = sizeof($schedule_days);
//
//            if ($schedule->type == 'One-Off') $number_of_occurrence = 1;
//
//            $payroll_exits = Payroll::where('payment_schedule_id','=',$schedule->id)
//                ->where('benefit_id','=',$schedule->benefit_id)->count();
//            if ($payroll_exits >= $number_of_occurrence) continue;
//
//            foreach ($schedule_days as $schedule_day) {
//
//                if ($schedule_day != date('d',strtotime(now()))) continue;
//
//                $this->generateNewPayrolls($schedule->benefit_id,$schedule_day);
//
//            }
//        }
//        dd($payrolls);

        return view('payment.payroll',compact('benefits','year','payrolls'));
//        return view('payment.payroll-list',compact('benefits'));
    }

    /**
     * Get the list of Application DataTable
     * @return mixed
     * @throws \Exception
     */
    public function getPayrolls()
    {
        $payrolls = Payroll::orderBy('created_at','desc')->get();

        return DataTables::of($payrolls)
            ->addColumn('action', function ($payroll) {
                $view = '<a href="'.route('payment.viewPayroll',[cryptThis($payroll->id)]).'"
                    class="dropdown-item" title="'. __("View Payroll") .'"><i class="la la-eye mr-2"></i> '. __("View Payroll") .'</a>';
                $clear = '';
                if ($payroll->rejected == 1) {
                    $view = '<a href="'.route('payment.viewPayroll',[cryptThis($payroll->id)]).'"
                    class="dropdown-item" title="'. __("View Payroll") .'"><i class="la la-eye mr-2"></i> '. __("View Payroll") .'</a>';
                }
                elseif ($payroll->cleared == 0 && endVerification($payroll->stage_lookup_id)) {

                    $clear = '<a href="'.route('payment.paidPayroll',[cryptThis($payroll->id)]).'"
                        class="dropdown-item" title="'. __("Record Payment") .'"
                        onclick="return confirm(\'Do you confirm record for this Payroll?\')">
                        <i class="la la-check-double mr-2"></i> '. __("Record Payment") .'</a>';
//                    return $view." ".$clear;
                }
                elseif ($payroll->cleared == 1) {
//                    $clear = '<a href="javascript:;" class="btn btn-icon btn-light-info btn-sm ml-1" data-toggle="tooltip" title="Initiate Refund"><i class="la la-donate"></i></a>';
//                    return $view." ".$clear;
//                    return $view;
                }
                elseif (startVerification($payroll->stage_lookup_id)) {
                    $view = '<a href="'.route('payment.viewPayroll',[cryptThis($payroll->id)]).
                            '" class="dropdown-item" data-toggle="tooltip" title="'. __("Validate") .'"><i class="la la-check mr-2"></i> '. __("Validate") .'</a>';

//                        <a href="'.route('payment.deletePayroll',[cryptThis($payroll->id)]).
//                            '" onclick="return confirm(\'Are you sure you want to delete this Payroll?\')"
//                        class="btn btn-icon btn-sm btn-danger" data-toggle="tooltip" title="'. __("Delete Payroll").'"><i class="la la-trash"></i></a>';
                    }
                else {
                    $view = '<a href="'.route('payment.viewPayroll',[cryptThis($payroll->id)]).
                        '" class="dropdown-item" data-toggle="tooltip" title="'. __("Approve") .'"><i class="la la-check mr-2"></i> '. __("Approve") .'</a>';
                }

                $options = '<div class="dropdown dropdown-inline mr-4">
                                <button type="button" class="btn btn-light-primary btn-icon btn-xs mb-0" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false">
                                    <i class="ki ki-bold-more-hor"></i>
                                </button>
                                <div class="dropdown-menu drop-left">'.$view. $clear.'</div>
                            </div>';

                return "$options";

            })
//            ->editColumn('benefit', function ($payroll) {
//                return $payroll->benefit->titles;
//            })
            ->editColumn('account', function ($payroll) {
                return $payroll->benefit->titles;
            })
            ->editColumn('category', function ($payroll) {
                return $payroll->benefit->titles;
            })
            ->editColumn('amount', function ($payroll) {
                if ($payroll->cleared == 1) {
                    return '<span class="text-success font-weight-bolder text-center">PAID</span>';
                }
                else {
                    return getVerificationLabel($payroll->stage_lookup_id);
                }
            })
            ->editColumn('status', function ($payroll) {
                if ($payroll->cleared == 1) {
                    return '<span class="text-success font-weight-bolder text-center">PAID</span>';
                }
                else {
                    return getVerificationLabel($payroll->stage_lookup_id);
                }
            })
            ->editColumn('period', function ($payroll) {
                //payment_date refers to the payroll generation date
                return date('d-M-Y',strtotime($payroll->payment_date));
            })
            ->editColumn('date', function ($payroll) {
                //payment_date refers to the payroll generation date
                return date('d-M-Y H:i',strtotime($payroll->payment_date));
            })
            ->escapeColumns([])
            ->make();
    }

    public function createPayroll(Request $request)
    {
        $benefit = Benefit::find($request->benefit);
        $page_title = 'New Payroll';
        $schedule = $benefit->paymentSchedule;
        $export_titles = $benefit->titles." Payroll Voucher";
        $header_title = $benefit->titles." Voucher: ";

        return view('payment.create-payroll',compact('benefit','schedule','page_title','export_titles','header_title'));
    }

    public function editPayroll($payroll_id)
    {
        $payroll = Payroll::find(Crypt::decrypt($payroll_id));
        $benefit = $payroll->benefit;
        $schedule = $benefit->paymentSchedule;
        $page_title = 'View Payroll';

        $form_mapping = BeneficiariesFormMapping::where('form_id','=', $benefit->form->id)->first();
        $form_fields_to_show = Field::whereIn('id',$form_mapping->show_fields_on_table)->orderBy('order','asc')->get();

        $beneficiaries = Beneficiary::where('form_id','=',$benefit->form->id)->where('archived','=',0)->get();

        return view('payment.create-payroll',compact('benefit','schedule','payroll','page_title',
            'beneficiaries','form_fields_to_show'));
    }

    public function viewPayroll($payroll_id)
    {
        $payroll = Payroll::find(Crypt::decrypt($payroll_id));
        $benefit = $payroll->benefit;
        $schedule = $benefit->paymentSchedule;
        $page_title = 'View Payroll';
        $export_titles = $benefit->titles." Payroll Voucher";
        $date = new \DateTime($payroll->payment_date);
        $date->modify('last day of this month');

        $header_title = $benefit->titles." Voucher: ".date('d-M-Y', strtotime($payroll->payment_date));
//        $header_title = $benefit->titles." Voucher: ".date('d-M-Y', strtotime($payroll->payment_date))." - ".$date->format('Y-m-d');

        //Get the column options for the Christmas Bonus
        if ($benefit->id == config('settings.christmas_bonus_id')) {
            $benefits_forms = Benefit::where('main_benefit','=', 1)->where('archived','=',0)->pluck('id')->toArray();
            $beneficiaries = Application::where('is_beneficiary','=',1)->whereIn('benefit_id',$benefits_forms)
                ->where('archived','=',0)
                ->whereIn('id',$payroll->beneficiaries_id)
                ->get();
            $form_fields_to_show = Field::where('form_id','=',config('settings.beneficiary_details_form_id'))->orderBy('order','asc')->get();
        }
    else {
            $form_mapping = BeneficiariesFormMapping::where('form_id','=', $benefit->form->id)->first();
            $form_fields_to_show = Field::whereIn('id',$form_mapping->show_fields_on_table)->orderBy('order','asc')->get();

            $beneficiaries = Application::where('is_beneficiary','=',1)->where('benefit_id','=',$benefit->id)
                ->where('archived','=',0)
                ->whereIn('id',$payroll->beneficiaries_id)
                ->get();
        }
//        dd($beneficiaries);

        $payment_schedule = $benefit->paymentSchedule;
        $status_information = getVerificationStatusData($payroll->stage_lookup_id);

        $show_list = true;
        if (isset($status_information)) {
            if (endVerificationByRejecting($status_information->id)) {
                $show_list = false;
            }
        }

        $verification_processes = LookupCode::activePayrollProcessOptions();
        $is_a_process = $benefit->require_processing;
//        $is_a_process = $this->isProcessForm($benefit->form->id); //Previous check

        $permitted_roles = explode(',',$payroll->responsible_role);
        $permitted = false;
        if (in_array(Auth::user()->roles->pluck('id')->first(),$permitted_roles)) {
            $permitted = true;
        }

        PayrollViewLog::create([
           'user_id'    =>  Auth::id(),
           'payroll_id'    =>  $payroll->id
        ]);

        return view('payment.create-payroll',compact('benefit','schedule','payroll','page_title',
            'beneficiaries','form_fields_to_show','payment_schedule','status_information','permitted','is_a_process',
            'verification_processes','export_titles','header_title','show_list'));
    }

    /**
     * Get the list of Beneficiaries on the Benefit Payroll DataTable
     * @param $benefit_id
     *
     * @return mixed
     * @throws \Exception
     */
    public function getPayrollBeneficiaries($benefit_id)
    {
        $benefit = Benefit::find(Crypt::decrypt($benefit_id));
        $beneficiaries = Beneficiary::where('form_id','=',$benefit->form_id)->where('archived','=',0)->get();

        return DataTables::of($beneficiaries)
            ->addColumn('action', function ($payroll) {
                if (!isset($payroll->approved_by)) {
                    return '<a href="'.route('payment.editPayroll',[cryptThis($payroll->id)]).
                        '" class="btn btn-icon btn-sm btn-light-primary" data-toggle="tooltip" title="'. __("Edit Payroll") .'"><i class="la la-edit"></i></a>

                        <a href="'.route('payment.viewPayroll',[cryptThis($payroll->id)]).
                        '" class="btn btn-sm btn-success" data-toggle="tooltip" title="'. __("View & Submit Payroll for Approval") .'"><i class="la la-money-check"></i>'. __("View & Submit Payroll") .'</a>

                        <a href="'.route('payment.deletePayroll',[cryptThis($payroll->id)]).
                        '" onclick="return confirm(\'Are you sure you want to delete this Payroll?\')"
                        class="btn btn-icon btn-sm btn-danger" data-toggle="tooltip" title="'. __("Delete Payroll").'"><i class="la la-trash"></i></a>';
                }
                else {
                    return '<a href="'.route('payment.viewPayroll',[cryptThis($payroll->id)]).'"
                    class="btn btn-primary" data-toggle="tooltip" title="'. __("View Payroll") .'"><i class="la la-box-open"></i> '. __("Open") .'</a>';
                }

            })
            ->editColumn('benefit', function ($payroll) {
                return $payroll->benefit->titles;
            })
            ->editColumn('status', function ($payroll) {
                return getVerificationLabel($payroll->status);
            })
            ->editColumn('payment_date', function ($payroll) {
                return date('d M, Y',strtotime($payroll->payment_date));
            })
            ->escapeColumns([])
            ->make();
    }

    public function savePayroll(Request $request)
    {
        $this->validate($request,[
            'benefit_id' => 'required|integer|min:1',
            'payment_date' => 'required|integer|min:1'
        ]);

        $benefit = Benefit::find($request->benefit_id);
        $beneficiaries = $this->getUpdatedBeneficiaries(cryptThis($benefit->id));

        try{
            if (isset($request->payroll_id)) {
                $check_payroll = Payroll::where('payment_date','=',date_create(date('Y-m-').$request->payment_date))
                    ->where('benefit_id','=',$request->benefit_id)->pluck('id');
                if(!in_array(Crypt::decrypt($request->payroll_id), $check_payroll->toArray()) && sizeof($check_payroll->toArray()) > 0) {
                    alertNotify(__('Sorry! This payroll already exists.'), null, 'danger');
                    return back();
                }

                $owner = Payroll::updateOrCreate([
                    'id'                    =>  Crypt::decrypt($request->payroll_id),
                ],[
                    'payment_date'          =>  date_create(date('Y-m-').$request->payment_date),
                    'benefit_id'            =>  $request->benefit_id,
                    'payment_schedule_id'   =>  $benefit->paymentSchedule->id,
                    'created_by'            =>  Auth::id(),
                    'beneficiaries_id'      => $beneficiaries
                ]);
            }
            else {

                $owner = Payroll::updateOrCreate([
                    'payment_date'          =>  date_create(date('Y-m-').$request->payment_date),
                    'benefit_id'   =>  $request->benefit_id,
                ],[
                    'payment_schedule_id'   =>  $benefit->paymentSchedule->id,
                    'created_by'            =>  Auth::id(),
                    'beneficiaries_id'      => $beneficiaries
                ]);
            }

            if ($owner) {
                alertNotify(__('Payroll has been saved!'), null, 'success');
            }
            else {
                alertNotify(__('Sorry! Something went wrong. Try again'), null, 'danger');
            }

        }catch(\Exception $ex){
            alertNotify(__($ex->getMessage()), null, 'warning');
            return back();
        }

        return redirect()->route('payment.payroll');
    }

    public function generateNewPayrolls($benefit_id,$payment_day,$payroll_id=null)
    {
        $benefit = Benefit::find($benefit_id);
//        $payment_day = null;
        $beneficiaries = $this->getUpdatedBeneficiaries(cryptThis($benefit_id));

        try{
            if (isset($payroll_id)) {

                $check_payroll = Payroll::where('payment_date','=',date_create(date('Y-m-').$payment_day))
                    ->where('benefit_id','=',$benefit_id)->pluck('id');

                if(!in_array(Crypt::decrypt($payroll_id), $check_payroll->toArray()) && sizeof($check_payroll->toArray()) > 0) {
                    alertNotify(__('Sorry! This payroll already exists.'), null, 'danger');
                    return back();
                }

                $payroll = Payroll::updateOrCreate([
                    'id'                    =>  Crypt::decrypt($payroll_id),
                ],[
                    'payment_date'          =>  date_create(date('Y-m-').$payment_day),
                    'benefit_id'            =>  $benefit_id,
                    'payment_schedule_id'   =>  $benefit->paymentSchedule->id,
                    'created_by'            =>  Auth::id(),
                    'beneficiaries_id'      => $beneficiaries
                ]);
            }
            else {

                $payroll = Payroll::updateOrCreate([
                    'payment_date'          =>  date_create(date('Y-m-').$payment_day),
                    'benefit_id'   =>  $benefit_id,
                ],[
                    'payment_schedule_id'   =>  $benefit->paymentSchedule->id,
                    'created_by'            =>  Auth::id(),
                    'beneficiaries_id'      => $beneficiaries
                ]);
            }

            if ($payroll) {
                $this->movePayroll($payroll->id,'Payroll has been generated.',$next_status=null);
                return true;
            }
            else {
                return false;
            }

        }catch(\Exception $ex){
            return false;
        }
    }

    public function getUpdatedBeneficiaries($benefit_id)
    {
        $beneficiaries_list = array();
        $beneficiaries_number = array();

        if (Crypt::decrypt($benefit_id) == config('settings.christmas_bonus_id')) {
            $benefits = Benefit::where('main_benefit','=', 1)->where('archived','=',0)->get();
        }
        else {
            $benefits = Benefit::where('id','=',Crypt::decrypt($benefit_id))->get();
        }

        foreach ($benefits as $benefit) {
            $beneficiaries = Application::where('benefit_id','=',$benefit->id)->where('is_beneficiary','=',1)->where('archived','=',0)->get();
            if((sizeof($beneficiaries)) < 1) continue;

            foreach ($beneficiaries as $beneficiary) {
                if (applicant_eligible_benefits($beneficiary->applicant_number)) {
                    if (in_array($beneficiary->applicant_number,$beneficiaries_number)) {
                        continue;
                    }
                    else {
                        $beneficiaries_number[] = $beneficiary->applicant_number;
                    }
                    $beneficiaries_list[] = $beneficiary->id;
                }
            }
        }

        return $beneficiaries_list;
    }

    public function refreshPayroll($benefit_id,$payroll_id)
    {
        $payroll = Payroll::find(Crypt::decrypt($payroll_id));
        $beneficiaries = $this->getUpdatedBeneficiaries($benefit_id);
        $payroll->beneficiaries_id = $beneficiaries;
        if ($payroll->save()){
            alertNotify(__('Payroll has been updated'), null, 'success');
        }
        else {
            alertNotify(__('Sorry! Something went wrong. Please try again.'), null, 'danger');
        }
        return back();
    }

    public function PaidPayroll($payroll_id)
    {
        $payroll = Payroll::find(Crypt::decrypt($payroll_id));
        $payroll->approved_by = Auth::id();
        $payroll->cleared = true;
//        dd($payroll->benefit->form->table_name." ".config('settings.funeral_allowance'));

        $beneficiaries = $payroll->beneficiaries_id;
        $beneficiaries_data = array();
        if($payroll->benefit->form->table_name == config('settings.pension_benefit')) {
            foreach ($beneficiaries as $key=>$beneficiary) {
                $getAmount = getBeneficiaryAgeGroupTaxByID($beneficiary,$payroll->benefit_id);
                $beneficiaries_data[$beneficiary] = $getAmount['amount']*((100 - $getAmount['tax'])/100);
            }
        }
        elseif($payroll->benefit->form->table_name == config('settings.funeral_allowance')) {
            foreach ($beneficiaries as $key=>$beneficiary) {
                $getAmount = getBeneficiaryAgeGroupTaxByID($beneficiary,$payroll->benefit_id);
                $beneficiaries_data[$beneficiary] = $getAmount['amount']*((100 - $getAmount['tax'])/100);
            }
        }
        else {
            foreach ($beneficiaries as $key=>$beneficiary) {
                $beneficiaries_data[$beneficiary] = $payroll->paymentSchedule->amount*((100 - $payroll->paymentSchedule->tax)/100);;
            }
        }
//        dd($beneficiaries_data);
        if ($payroll->save()){
            PayrollHistory::updateOrCreate([
                'payroll_id'    =>  Crypt::decrypt($payroll_id)
            ],[
                'benefit_id'    =>  $payroll->benefit_id,
                'confirmed_by'    =>  Auth::id(),
                'beneficiaries_data'    =>  $beneficiaries_data,
            ]);

            PayrollVerificationProcessLog::create([
                'payroll_id'                =>  $payroll->id,
                'user_id'                   =>  Auth::id(),
                'verification_process_id'   =>  $payroll->stage_lookup_id,
                'message'                   =>  "Payroll payment has been made and cleared."
            ]);

            alertNotify(__('Payroll has been Paid and Cleared'), null, 'success');
        }
        else {
            alertNotify(__('Sorry! Something went wrong. Please try again.'), null, 'danger');
        }
        return back();
    }

    public function submitPayroll(Request $request, $payroll_id)
    {
        $payroll = Payroll::find(Crypt::decrypt($payroll_id));
        $submit_id = getStartProcessCode(config('settings.payment_process_lookupCode'));

        $next_status = 0;
        $message = "";
        if ($request->next_action == 'proceed') {
            $next_status = getNextStage($payroll->stage_lookup_id);
        }
        elseif ($request->next_action == 'return') {
            $message = "Action: Returned <br> ";
            $next_status = getPreviousStage($payroll->stage_lookup_id);
        }
        elseif ($request->next_action == 'reject') {
            $next_status = getDeclinedStage($payroll->stage_lookup_id);
        }

        if (startVerification($submit_id)) {
            $beneficiaries = $this->getUpdatedBeneficiaries(cryptThis($payroll->benefit->id));
            $payroll->beneficiaries_id = $beneficiaries;
        }
//        $message = "Payroll Status has been updated.<br>";
        if (isset($next_status) or !is_null($next_status)) {
            $submit_id = $next_status;
            $message .= $request->message;
        }
        if (endVerificationByRejecting($next_status)) {
            $payroll->rejected = 1;
            $message = "Payroll has been rejected.<br>".$request->message;
        }

        if ($request->message == "" || $request->message == null) {
            alertNotify(__('Please enter a feedback message before you can proceed.'), null, 'warning');
            return back();
        }

        $payroll->stage_lookup_id = $submit_id;
        $payroll->status = 1;
        if ($payroll->save()) {

            $getRole = StageEmail::where('lookup_id','=',$submit_id)->pluck('roles')->first();
            $payroll->responsible_role = $getRole;
            $payroll->save();

            if ($payroll->save()) {
                PayrollVerificationProcessLog::create([
                    'payroll_id'                =>  $payroll->id,
                    'user_id'                   =>  Auth::id(),
                    'verification_process_id'   =>  $submit_id,
                    'message'                   =>  $message
                ]);

                //Send Email Notification
                $this->send_email($submit_id,'payrolls',Crypt::decrypt($payroll_id));
            }

            alertNotify(__('Payroll has been submitted for approval'), null, 'success');
        }
        else {
            alertNotify(__('Sorry! Something went wrong. Please try again.'), null, 'danger');
            return back();
        }
        return redirect()->route('payment.payroll');
    }

    public function movePayroll($payroll_id,$message,$next_status=null)
    {
        $payroll = Payroll::find($payroll_id);
        $submit_id = getStartProcessCode(config('settings.payment_process_lookupCode'));

        if (startVerification($submit_id)) {
            $beneficiaries = $this->getUpdatedBeneficiaries(cryptThis($payroll->benefit->id));
            $payroll->beneficiaries_id = $beneficiaries;
        }

        if (isset($next_status) or !is_null($next_status)) {
            $submit_id = $next_status;

            if (endVerificationByRejecting($submit_id)) {
                $payroll->rejected = 1;
            }
        }

        $payroll->stage_lookup_id = $submit_id;
        $payroll->status = 1;
        if ($payroll->save()) {

            $getRole = StageEmail::where('lookup_id','=',$submit_id)->pluck('roles')->first();
            $payroll->responsible_role = $getRole;
            $payroll->save();

            if ($payroll->save()) {
                PayrollVerificationProcessLog::create([
                    'payroll_id'                =>  $payroll->id,
                    'user_id'                   =>  Auth::id(),
                    'verification_process_id'   =>  $submit_id,
                    'message'                   =>  $message
                ]);

                //Send Email Notification
                $this->send_email($submit_id,'payrolls',$payroll_id);
            }

            return true;
        }
        return false;
    }

    public function deletePayroll($id)
    {
        try {
            $payroll_id = Crypt::decrypt($id);

            if (Payroll::destroy($payroll_id)) {
                alertNotify(__('Payroll has been deleted!'), null, 'success');
            }
            else {
                alertNotify(__('Sorry! Something went wrong. Please try again.'), null, 'danger');
            }
        }catch (\Exception $exception){
            alertNotify(__($exception->getMessage()), null, 'warning');
        }
        return back();
    }

    /**
     * Debt Recovery
     */
    public function debtRecovery()
    {
        $debt_recoveries = DebtRecovery::all();
        $partial_debt_recoveries = DebtRecovery::whereRaw('amount_due > amount_paid')->get();
        $debt_recovered = DebtRecovery::whereRaw('amount_due <= amount_paid')->get();
        return view('payment.debt-recovery-list',compact('debt_recoveries','partial_debt_recoveries','debt_recovered'));
    }

    public function editDebtRecovery($id)
    {
        $debt_recovery = DebtRecovery::find(Crypt::decrypt($id));
        return view('payment.edit-debt-recovery',compact('debt_recovery'));
    }

    public function updateDebtRecovery(Request $request,$id)
    {
        $request->validate([
           'commencement_date'  =>  'required|date',
           'expiry_date'  =>  'required|date',
//           'amount_paid'  =>  'required|numeric|min:0',
           'payment_plan'  =>  'required|string|min:2',
        ]);
        $debt_recovery = DebtRecovery::find(Crypt::decrypt($id));
        $debt_recovery->commencement_date = date('Y-m-d', strtotime($request->commencement_date));
        $debt_recovery->expiry_date = date('Y-m-d', strtotime($request->expiry_date));
//        $debt_recovery->amount_paid = $request->amount_paid;
        $debt_recovery->payment_plan = $request->payment_plan;
        $debt_recovery->last_edited_by = Auth::id();

        if ($debt_recovery->save()) {
            alertNotify('Details has been saved.','','success');
        }
        else {
            alertNotify('Sorry! Please try again.','','danger');
            return back();
        }

        return back();
    }

    public function updateDebtRecoveryAmount(Request $request,$id)
    {
        $debt_recovery = DebtRecovery::find(Crypt::decrypt($id));
        $amount_left_to_pay = $debt_recovery->amount_due - $debt_recovery->paidAmounts->sum('amount');
        $request->validate([
           'amount_paid'  =>  'required|numeric|min:0|max:'.$amount_left_to_pay,
           'remark'  =>  'required|string|min:2',
        ]);

        if ($request->amount_paid > $debt_recovery->amount_due - $debt_recovery->paidAmounts->sum('amount')) {
            alertNotify('Amount paid exceed the amount left to be paid, $'.$amount_left_to_pay,'', 'warning');
            return back();
        }

        $file_upload=null;
        if($request->file('file_upload')) {
            $directory = 'public/debt-recovery-receipts/'.Crypt::decrypt($id);
            Storage::putFileAs("$directory", $request->file('file_upload'), $request->file_upload->getClientOriginalName());
            $file_upload = $directory.'/'.$request->file_upload->getClientOriginalName();
        }

        $debt_recovery_history = DebtRecoveryAmountPaidHistory::create([
            'user_id'       =>  Auth::id(),
            'debt_recovery_id'       =>  Crypt::decrypt($id),
            'amount'       =>  $request->amount_paid,
            'remark'       =>  $request->remark,
            'receipt'       =>  $file_upload
        ]);

        if ($debt_recovery_history) {
            $debt_recovery->amount_paid += $request->amount_paid;
            $debt_recovery->save();
            alertNotify('Payment recorded.','','success');
        }
        else {
            alertNotify('Sorry! Please try again.','','danger');
            return back();
        }

        return redirect()->route('payment.debtRecovery');
    }

    public function deleteDebtRecovery($id)
    {
        try {
            $debt_id = $id;

            if (DebtRecovery::destroy($debt_id)) {
                alertNotify(__('Beneficiary Debt has been deleted!'), null, 'success');
            }
            else {
                alertNotify(__('Sorry! Something went wrong. Please try again.'), null, 'danger');
            }
        }catch (\Exception $exception){
            alertNotify(__($exception->getMessage()), null, 'warning');
        }
        return back();
    }

    /**
     * @param $report_values
     * @param $baseline_values
     * @param $target_values
     * @param $years
     *
     * @throws \PhpOffice\PhpSpreadsheet\Exception
     */
    private function generalspreadsheet($report_values, $baseline_values, $target_values,$years)
    {
        $spreadsheet = new Spreadsheet();

        $allStyleArray = [
            'alignment' => [
                'wrapText'  => true,
                'vertical'  =>  Alignment::VERTICAL_TOP,
            ],
        ];
        $figuresStyleArray = [
            'alignment' => [
                'horizontal' => Alignment::HORIZONTAL_RIGHT,
            ],
        ];
        $styleArray = [
            'font' => [
                'bold' => true,
            ],
        ];

        $sheet = $spreadsheet->getActiveSheet();
        $sheet->getDefaultColumnDimension()->setWidth(12);
        $sheet->getColumnDimension('A')->setWidth(50);
        $sheet->getColumnDimension('B')->setWidth(5);
        $sheet->getColumnDimension('C')->setWidth(50);
        $sheet->getColumnDimension('D')->setWidth(30);
        $sheet->getStyle('A1')->getAlignment()->setWrapText(true);

        $sheet->mergeCells('A1:A2');
        $sheet->setCellValue('A1', __('ACE Level Results Indicators'))->getStyle('A1')->applyFromArray($styleArray);
        $sheet->mergeCells('B1:B2');
        $sheet->setCellValue('B1', __('Core'))->getStyle('B1')->applyFromArray($styleArray);
        $sheet->mergeCells('C1:C2');
        $sheet->setCellValue('C1', __('Unit of Measure'))->getStyle('C1')->applyFromArray($styleArray);
        $sheet->mergeCells('D1:D2');
        $sheet->setCellValue('D1', __('Specifics'))->getStyle('D1')->applyFromArray($styleArray);
        $sheet->mergeCells('E1:E2');
        $sheet->setCellValue('E1', __('Baseline'))->getStyle('E1')->applyFromArray($styleArray);

        $active_row = 1;
        $target_start_column = 6;
        $years_range = sizeof($years);

        /**
         * For Annual Target Values Header
         */
        $sheet->mergeCellsByColumnAndRow($target_start_column,$active_row,$target_start_column + $years_range - 1,$active_row)
            ->setCellValueByColumnAndRow($target_start_column,$active_row,__('Annual Target Values'))
            ->getStyleByColumnAndRow($target_start_column,$active_row)->applyFromArray($styleArray);

        /**
         * For Dynamic annual Target Values
         */
        $year_number = ['2019'=>1,'2020'=>2,'2021'=>3,'2022'=>4,'2023'=>5];
        $second_row = $active_row + 1;
        for($a = 0; $a < sizeof($years); $a++) {
            $sheet->setCellValueByColumnAndRow($target_start_column + $a, $second_row,__("Year ").$year_number[$years[$a]])
                ->getStyleByColumnAndRow($target_start_column + $a,$second_row)->applyFromArray($styleArray);
        }

        /**
         * For Annual Results
         */
        $next_column = $result_start_column = $target_start_column + $years_range;
        for($a = 0; $a < sizeof($years); $a++) {
            $sheet->mergeCellsByColumnAndRow($result_start_column + $a,$active_row,$result_start_column + $a,$active_row + 1)
                ->setCellValueByColumnAndRow($result_start_column + $a, $active_row,$years[$a]." ".__('Results'))
                ->getStyleByColumnAndRow($result_start_column + $a,$active_row)->applyFromArray($styleArray);
            $next_column += 1;
        }

        /**
         * For Total Results
         */
        $sheet->mergeCellsByColumnAndRow($next_column,$active_row,$next_column,$active_row+1)
            ->setCellValueByColumnAndRow($next_column,$active_row,__('Total Results'))
            ->getStyleByColumnAndRow($next_column,$active_row)->applyFromArray($styleArray);

        $row = 3; $next_column = 1;
        $sheet = $spreadsheet->getActiveSheet();
        $indicators = Indicator::where('is_parent','=',1)
            ->where('status','=',1)
            ->where('show_on_report','=',1)
            ->orderBy('order_on_report','asc')->get();

        foreach ($indicators as $indicator) {
            $row_counter = $indicator->indicators->where('show_on_report','=',1)->count();
            $next_column = 1;

            /**
             * Column: ACE Level Results Indicators
             */
            if ($row_counter > 1) {
                $sheet->mergeCellsByColumnAndRow($next_column,$row,$next_column,$row + $row_counter - 1)
                    ->setCellValueByColumnAndRow($next_column,$row,$indicator->identifier.": ".$indicator->title)
                    ->getStyleByColumnAndRow($next_column,$row)->applyFromArray($allStyleArray);
            }
            else {
                $sheet->setCellValueByColumnAndRow($next_column,$row,$indicator->identifier.": ".$indicator->title)
                    ->getStyleByColumnAndRow($next_column,$row)->applyFromArray($allStyleArray);
            }
            $next_column += 1;

            /**
             * Column: Core
             */
            if ($row_counter > 1) {
                $sheet->mergeCellsByColumnAndRow($next_column,$row,$next_column,$row + $row_counter - 1)
                    ->setCellValueByColumnAndRow($next_column,$row," ");
            }
            else {
                $sheet->setCellValueByColumnAndRow($next_column,$row," ");
            }
            $next_column += 1;

            /**
             * Column: Unit of Measure
             */
            if ($row_counter > 1) {
                $sheet->mergeCellsByColumnAndRow($next_column,$row,$next_column,$row + $row_counter - 1)
                    ->setCellValueByColumnAndRow($next_column,$row,$indicator->unit_measure)
                    ->getStyleByColumnAndRow($next_column,$row)->applyFromArray($allStyleArray);
            }
            else {
                $sheet->setCellValueByColumnAndRow($next_column,$row,$indicator->unit_measure)
                    ->getStyleByColumnAndRow($next_column,$row)->applyFromArray($allStyleArray);
            }
            $next_column += 1;

            $sub_indicators = $indicator->indicators->where('status', '=', 1)->where('show_on_report', '=', 1);
            $total_result = 0;$inner_row_counter = $row;

            if ($sub_indicators->count() > 0) {
                foreach ($sub_indicators as $sub_indicator) {
                    $inner_column_counter = $next_column;

                    /**
                     * Column: Specifics
                     */
                    $sheet->setCellValueByColumnAndRow($inner_column_counter,$inner_row_counter,$sub_indicator->title)
                        ->getStyleByColumnAndRow($inner_column_counter,$inner_row_counter)->applyFromArray($allStyleArray);
                    $inner_column_counter += 1;

                    /**
                     * Column: Baselines
                     */
                    if(sizeof($baseline_values) > 0 && array_key_exists($sub_indicator->id,$baseline_values)) {
                        $baseline = $baseline_values[$sub_indicator->id];
                    } else {
                        $baseline = 0;
                    }
                    $sheet->setCellValueByColumnAndRow($inner_column_counter,$inner_row_counter,$baseline)
                        ->getStyleByColumnAndRow($inner_column_counter,$inner_row_counter)->applyFromArray($figuresStyleArray);
                    $inner_column_counter += 1;

                    /**
                     * Column: Target Values
                     */
                    foreach($years as $key=>$year) {
                        if(sizeof($target_values["$year"]) > 0 && array_key_exists($sub_indicator->id,$target_values["$year"])) {
                            $target = $target_values["$year"][$sub_indicator->id];
                        } else {
                            $target = 0;
                        }
                        $sheet->setCellValueByColumnAndRow($inner_column_counter,$inner_row_counter,$target)
                            ->getStyleByColumnAndRow($inner_column_counter,$inner_row_counter)->applyFromArray($figuresStyleArray);
                        $inner_column_counter += 1;
                    }

                    /**
                     * Column: Report Results
                     */
                    foreach($years as $key=>$year) {
                        try{
                            $value = $report_values["$year"]->where('indicator_id','=',$sub_indicator->id)
                                ->pluck('ind_values')->first();
                            if ($value == "") {
                                $value = 0;
                            }
                            $total_result += (integer)$value;
                        }
                        catch(\Exception $exception){
                            $value = 0;
                        }
                        $sheet->setCellValueByColumnAndRow($inner_column_counter,$inner_row_counter,$value)
                            ->getStyleByColumnAndRow($inner_column_counter,$inner_row_counter)->applyFromArray($figuresStyleArray);
                        $inner_column_counter += 1;
                    }

                    /**
                     * Column: total Result
                     */
                    $sheet->setCellValueByColumnAndRow($inner_column_counter,$inner_row_counter,$total_result)
                        ->getStyleByColumnAndRow($inner_column_counter,$inner_row_counter)->applyFromArray($figuresStyleArray);

                    $inner_row_counter += 1;
                }
            } else {
                /**
                 * Column: Specifics
                 */
                $sheet->setCellValueByColumnAndRow($next_column,$row,' ');
                $next_column += 1;

                /**
                 * Column: Baselines
                 */
                if(sizeof($baseline_values) > 0 && array_key_exists($indicator->id,$baseline_values)) {
                    $baseline = $baseline_values[$indicator->id];
                } else {
                    $baseline = 0;
                }
                $sheet->setCellValueByColumnAndRow($next_column,$row,$baseline)
                    ->getStyleByColumnAndRow($next_column,$row)->applyFromArray($figuresStyleArray);
                $next_column += 1;

                /**
                 * Column: Target Values
                 */
                foreach($years as $key=>$year) {
                    if(sizeof($target_values["$year"]) > 0 && array_key_exists($indicator->id,$target_values["$year"])) {
                        $target = $target_values["$year"][$indicator->id];
                    } else {
                        $target = 0;
                    }
                    $sheet->setCellValueByColumnAndRow($next_column,$row,$target)
                        ->getStyleByColumnAndRow($next_column,$row)->applyFromArray($figuresStyleArray);
                    $next_column += 1;
                }

                /**
                 * Column: Report Results
                 */
                foreach($years as $key=>$year) {
                    try{
                        $value = $report_values["$year"]->where('indicator_id','=',$indicator->id)
                            ->pluck('ind_values')->first();
                        if ($value == "") {
                            $value = 0;
                        }
                        $total_result += (integer)$value;
                    }
                    catch(\Exception $exception){
                        $value = 0;
                    }
                    $sheet->setCellValueByColumnAndRow($next_column,$row,$value)
                        ->getStyleByColumnAndRow($next_column,$row)->applyFromArray($figuresStyleArray);
                    $next_column += 1;
                }

                /**
                 * Column: total Result
                 */
                $sheet->setCellValueByColumnAndRow($next_column,$row,$total_result)
                    ->getStyleByColumnAndRow($next_column,$row)->applyFromArray($figuresStyleArray);
            }

            if ($row_counter < 1) {
                $row += 1;
            } else {
                $row += $row_counter;
            }

        }

        $writer = new Xls($spreadsheet);
        header('Content-Type: application/vnd.ms-excel');
        header('Content-Disposition: attachment;filename="'.__("Result Framework").'.xls"');
        header('Cache-Control: max-age=0');
        $writer->save('php://output');
    }
}
