<?php

namespace App\Http\Controllers;

use App\Models\Country;
use App\Models\CountryPlan;
use App\Models\CountryPlanPricing;
use App\Models\Module;
use App\Models\Plan;
use App\Models\PlanFeature;
use App\Models\PlanSubscription;
use Illuminate\Http\Request;
use Illuminate\Support\Arr;

class PlansSubscriptionController extends Controller
{

    public function list(Request $request)
    {
        $plans = Plan::orderBy('sort_order','asc');

        if (isset($request->country)) {
            $plans = $plans->where('country_id',$request->country);
        }

        if (isset($request->is_custom)) {
            $plans = $plans->where('is_custom',$request->is_custom);
        }

        $all_modules = array_merge(config('settings.plan_features'),config('settings.reporting_plan_features'),config('settings.support_plan_features'));

        $plans = $plans->get();
        $countries = Country::orderBy('country_name')->get();

        return view('plans.index', compact('plans','countries','all_modules'));
    }

    public function create()
    {
        $action = 'Create';
        return view('plans.create', compact('action'));
    }

    public function savePlan(Request $request)
    {
        $request->validate([
            'name'  =>  'required|string|min:3',
            'description'  =>  'required|string|min:3',
            'price'  =>  'required|integer|min:0',
            'year_discount_percent'  =>  'nullable|integer|min:0',
            'signup_fee'  =>  'required|numeric|min:0',
            'trial_period'  =>  'required|numeric|min:0|max:12',
            'active_subscribers_limit'  =>  'required|numeric|min:1',
            'grace_period'  =>  'nullable|numeric|min:0',
            'invoice_period'  =>  'nullable|numeric|min:1|max:12',
            'plan_features'  =>  'required|array',
        ],[
            'active_subscribers_limit.required' => 'Maximum Beneficiary limit required'
        ]);

        if (!$request->plan_slug) {
            $plan = Plan::create([
                'name' => $request->name,
                'description' => $request->description,
                'price' => $request->price,
                'year_discount_percent' => $request->year_discount_percent,
                'signup_fee' => $request->signup_fee,
                'invoice_period' => $request->invoice_period,
                'invoice_interval' => 'month',
                'trial_period' => $request->trial_period,
                'trial_interval' => 'month',
                'grace_period' => 0,
                'grace_interval' => 'day',
                'active_subscribers_limit' => $request->active_subscribers_limit,
                'sort_order' => $request->sort_order,
                'currency' => env('DEFAULT_CURRENCY_CODE'),
                'is_custom' => $request->is_custom,
            ]);
        }
        else {
            $plan = Plan::updateOrCreate(['slug'=>$request->plan_slug],[
                'name' => $request->name,
                'description' => $request->description,
                'price' => $request->price,
                'year_discount_percent' => $request->year_discount_percent,
                'signup_fee' => $request->signup_fee,
                'invoice_period' => $request->invoice_period,
                'invoice_interval' => 'month',
                'trial_period' => $request->trial_period,
                'trial_interval' => 'month',
                'grace_period' => 0,
                'grace_interval' => 'day',
                'active_subscribers_limit' => $request->active_subscribers_limit,
                'sort_order' => $request->sort_order,
                'is_custom' => $request->is_custom,
            ]);

            PlanFeature::wherePlanId($plan->id)->delete();
        }

        if ($plan) {
            foreach (config('settings.base_plan_features') as $key=>$value) {
                if (isset(config('settings.base_plan_features')[$key])) {
                    $plan->features()->saveMany([
                        new PlanFeature(config('settings.base_plan_features')[$key])
                    ]);
                }
            }

            foreach ($request->plan_features as $key=>$value) {
                if (isset(config('settings.plan_features')[$value])) {
                    $plan->features()->saveMany([
                        new PlanFeature(config('settings.plan_features')[$value])
                    ]);
                }
            }

            if (isset(config('settings.reporting_plan_features')[$request->reporting_plan_features])) {
                $plan->features()->saveMany([
                    new PlanFeature(config('settings.reporting_plan_features')[$request->reporting_plan_features])
                ]);
            }

            if (isset(config('settings.support_plan_features')[$request->support_plan_features])) {
                $plan->features()->saveMany([
                    new PlanFeature(config('settings.support_plan_features')[$request->support_plan_features])
                ]);
            }
            $plan->features()->whereNotNull('deleted_at')->forceDelete();

            $this->updateMenuWithFeatures($plan);

            notify('Plan information has been saved successfully','success','Saved');
            return redirect()->route('plan.list');
        }
        else {
            notify('Sorry! Something went wrong. Try again!','error','Error!');
            return  back();
        }

    }

    /**
     * @param Plan $plan
     * @return void
     */
    public function updateMenuWithFeatures(Plan $plan)
    {
        $deployments = PlanSubscription::byPlanId($plan->id)->findActive()
            ->where('subscriber_type','=','App\Models\Deployment')
            ->pluck('subscriber_id')
            ->toArray();

        $featuresMenu = $features = null;
        if (isset($plan->features)) {
            $featuresMenu = $plan->features->setVisible(['name','value','icon','route','description'])->toArray();
            $features = $plan->features->pluck('name')->toArray();
        }

        Module::whereIn('instance_id',array_values($deployments))->update([
            'modules'   =>  $features,
            'menus'   =>  $featuresMenu,
        ]);
    }

    public function editPlan($plan_slug)
    {
        $plan = Plan::where('slug',$plan_slug)->first();
        $countries = Country::orderBy('country_name')->get();

        if (!$plan) return back();
        $action = 'Edit';

        return view('plans.create', compact('plan','action','countries'));
    }

    public function saveCountryPlan(Request $request)
    {
        $request->validate([
            'country'   =>  'required|numeric',
            'currency'   =>  'required|numeric',
            'plan'   =>  'required|array',
        ]);

        $country_cost = CountryPlan::updateOrCreate([
            'country_id'    =>  $request->country
        ],[
            'currency_id'    =>  $request->currency
        ]);

        if ($country_cost) {
            foreach ($request->plan as $plan_id=>$pricing) {
                CountryPlanPricing::updateOrCreate([
                    'country_plan_id'   =>  $country_cost->id,
                    'country_id'   =>  $country_cost->country_id,
                    'plan_id'   =>  $plan_id,
                ],[
                    'pricing'   =>  $pricing
                ]);
            }

            notify('Country Plan has been saved successfully','success','Saved');
        }
        else {
            notify('Sorry! Something went wrong. Try again!','error','Error!');
        }
        return  back();
    }

    public function countryPricing(Request $request)
    {
        $countries = Country::has('regions', '>' , 0)->orderBy('country_name')->get();
        $country_plans = CountryPlan::with(['country','currency'])->get();
        return view('plans.country-pricing', compact('countries','country_plans'));
    }

    public function getCountryPricing($country_id)
    {
        $country_pricing = CountryPlan::with(['currency:id,currency_code','planPricing'=>['plan:id,name,year_discount_percent']])->where('country_id','=',$country_id)->first();
        if (isAdmin()) {
            $plans = Plan::all();
        }
        else {
            $plans = Plan::where('is_custom','!=',1)->get();
        }
        $options = view('components.country-plan-pricing', compact('country_pricing','plans'))->render();

        return response()->json(['options'=>$options]);
    }

    public function getIntervalPricing($plan_slug,$country_id)
    {
        $country_pricing = CountryPlan::with(['currency:id,currency_code','planPricing'=>['plan:id,name,year_discount_percent']])->where('country_id','=',$country_id)->first();
        $plan = Plan::findBySlug($plan_slug);
        $subscription_interval = view('components.plan-subscription-interval', compact('country_pricing','plan'))->render();

        return response()->json(['subscription_interval'=>$subscription_interval]);
    }

    public function getCountryPlan($country_plan_id)
    {
        $countries = Country::has('regions', '>' , 0)->orderBy('country_name')->get();

        if ($country_plan_id <= 0){
            $country_plan = null;
            $country_plan_pricing = [];
        }
        else {
            $country_plan = CountryPlan::find($country_plan_id);
            $country_plan_pricing = CountryPlanPricing::where('country_plan_id','=',$country_plan_id)->pluck('pricing','plan_id');
        }

        $plans = Plan::whereIsActive(true)->where('is_custom','=',0)->orWhereNull('is_custom')->get();
        $view = view('plans.country-plan-pricing',compact('country_plan','plans','countries','country_plan_pricing'))->render();
        return response()->json(['view'=>$view]);
    }

    public function deleteCountryPlan(Request $request)
    {
        $country_cost = CountryPlan::destroy($request->country_plan_id);

        if ($country_cost) {
            notify('Country Plan has been removed successfully','success','Saved');
        }
        else {
            notify('Sorry! Something went wrong. Try again!','error','Error!');
        }
        return  back();
    }
}
