<?php

namespace App\Http\Controllers;

use App\Http\Helpers\FormRender;
use App\Http\Traits\FormSubmission;
use App\Models\Project;
use App\Models\ProjectTimeline;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Crypt;
use Illuminate\Support\Facades\DB;
use Yajra\DataTables\DataTables;

class ProjectController extends Controller
{

    use FormSubmission;

    public function __construct()
    {
        $this->middleware(['auth', 'verified']);
    }

    /**
     * Display a listing of Projects.
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View
     */
    public function index()
    {
        return view('projects.list');
    }

    /**
     * Get the list of Projects DataTable
     * @return mixed
     * @throws \Exception
     */
    public function getProjectsList()
    {
        $projects = Project::where('archived','=',false)->orderBy('project_names','asc')->get();

        return DataTables::of($projects)
            ->addColumn('action', function ($project) {
                return '
                        <a href="'.route('project.projectTimeline',[cryptThis($project->id)]).
                    '" class="btn btn-sm btn-light-success mb-1" title="'. __("Project Timeline") .'"><i class="la la-list-alt"></i>'.__("Timeline").'</i></a>
                        <a href="'.route('project.edit',[cryptThis($project->id)]).
                    '" class="btn btn-sm btn-icon btn-light-primary mb-1" title="'. __("Edit Project") .'"><i class="la la-edit"></i></i></a>
                        <a href="'.route('project.archive',[cryptThis($project->id)]).'" 
                        onclick="return confirm(\'Archiving will prevent the project from receiving this benefit. Do you want to proceed?\')"
                        class="btn btn-sm btn-icon btn-warning mb-1" title="'. __("Archive Project") .'"><i class="la la-box"></i></a>
                        <a href="'.route('project.delete',[cryptThis($project->id)]).
                    '" onclick="return confirm(\'The project will be taken off the list completely. Do you want to proceed?\')" 
                        class="btn btn-sm btn-icon btn-danger mb-1" title="'. __("Delete Project").'"><i class="la la-trash"></i></a>';
            })
            ->editColumn('project_names', function ($project) {
                return $project->project_names;
            })
//            ->editColumn('locations', function ($project) {
//                return $project->locations;
//            })
            ->editColumn('project_owners', function ($project) {
                return $project->project_owners;
            })
            ->editColumn('project_costs', function ($project) {
                return money($project->project_costs);
            })
            ->editColumn('times', function ($project) {
                return $project->times;
            })
            ->editColumn('created_at', function ($project) {
                return date('d M, Y',strtotime($project->created_at));
            })
            ->escapeColumns([])
            ->make();
    }

    /**
     * List of archived Projects
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View
     */
    public function archives()
    {
        return view('projects.archived-list');
    }

    /**
     * Get the list of Projects DataTable
     * @return mixed
     * @throws \Exception
     */
    public function getArchivedProjectsList()
    {
        $projects = Project::where('archived','=',true)->orderBy('project_names','asc')->get();

        return DataTables::of($projects)
            ->addColumn('action', function ($project) {
                return '<a href="'.route('project.archive',[cryptThis($project->id)]).'" 
                        onclick="return confirm(\'Restoring this project will be eligible for Project. Do you want to proceed?\')"
                        class="btn btn-sm btn-icon btn-primary mb-1" title="'.__("Restore Project").'"><i class="la la-box-open"></i></a>
                        <a href="'.route('project.delete',[cryptThis($project->id)]).
                    '" onclick="return confirm(\'The project will be taken off the list completely . Do you want to proceed ? \')" 
                        class="btn btn-sm btn-icon btn-danger mb-1" title="'. __("Delete Project").'"><i class="la la-trash"></i></a>';
            })
            ->editColumn('project_names', function ($project) {
                return $project->project_names;
            })
            ->editColumn('locations', function ($project) {
                return $project->locations;
            })
            ->editColumn('project_owners', function ($project) {
                return $project->project_owners;
            })
            ->editColumn('project_costs', function ($project) {
                return money($project->project_costs);
            })
            ->editColumn('times', function ($project) {
                return $project->times;
            })
            ->editColumn('created_at', function ($project) {
                return date('d M, Y',strtotime($project->created_at));
            })
            ->escapeColumns([])
            ->make();
    }

    /**
     * Create Project information
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View
     * @throws \Throwable
     */
    public function createProject()
    {
        $get_render_form = new FormRender();
        $get_render_form->submitUrl(route('project.saveProject'));
        $render_form = $get_render_form->renderFormView('projects');
        $page_title = __('Create Project');
        return view('projects.edit', compact('render_form','page_title'));
    }

    /**
     * Save Project information
     * @param Request $request
     * @return \Illuminate\Http\RedirectResponse
     */
    public function saveProject(Request $request)
    {
        $project = $this->submit($request);

        if ($project) {
            $message = "The information has been updated.";
            $type = "success";
            alertNotify($message,null,$type);
        }
        return redirect()->route('project.list');
    }

    /**
     * Edit Project information
     * @param $project_id
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View
     * @throws \Throwable
     */
    public function edit($project_id)
    {
        $get_render_form = new FormRender();
        $get_render_form->request_id($project_id);
        $get_render_form->submitUrl(route('project.update',[$project_id]));
        $render_form = $get_render_form->renderFormView('projects');
        $page_title = __('Edit Details');
        return view('projects.edit', compact('render_form','page_title'));
    }

    /**
     * Update Project information
     * @param Request $request
     * @param $project_id
     * @return \Illuminate\Http\RedirectResponse
     */
    public function update(Request $request, $project_id)
    {
        $project = $this->submit($request);

        if ($project) {
            $message = "The information has been updated.";
            $type = "success";
            alertNotify($message,null,$type);
        }
        return redirect()->route('project.list');
    }

    /**
     * Project Timeline information
     * @param $project_id
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View
     * @throws \Throwable
     */
    public function projectTimeline($project_id)
    {
        $project = Project::find(Crypt::decrypt($project_id));
        $page_title = __('Project Timeline').' - '.$project->project_names;
        return view('projects.timeline', compact('page_title','project','project_id'));
    }

    /**
     * Project Timeline information
     * @param $project_id
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View
     * @throws \Throwable
     */
    public function createProjectTimeline($project_id)
    {
        $project = Project::find(Crypt::decrypt($project_id));
        $get_render_form = new FormRender();
//        $get_render_form->request_id($project_id);
        $get_render_form->submitUrl(route('project.saveProjectTimeline',[$project_id]));
        $render_form = $get_render_form->renderFormView('project_timelines');
        $page_title = __('Project Timeline').' - '.$project->project_names;
        return view('projects.create-timeline', compact('render_form','page_title','project','project_id'));
    }

    /**
     * Edit Project Timeline information
     * @param $project_id
     * @param $timeline_id
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View
     * @throws \Throwable
     */
    public function editProjectTimeline($project_id,$timeline_id)
    {
        $project = Project::find(Crypt::decrypt($project_id));
        $get_render_form = new FormRender();
        $get_render_form->request_id($timeline_id);
        $get_render_form->submitUrl(route('project.updateProjectTimeline',[$project_id,$timeline_id]));
        $render_form = $get_render_form->renderFormView('project_timelines');
        $page_title = __('Edit Project Timeline').' - '.$project->project_names;
        return view('projects.create-timeline', compact('render_form','page_title','project','project_id'));
    }

    /**
     * Save Project Timeline information
     * @param Request $request
     * @param $project_id
     * @return \Illuminate\Http\RedirectResponse
     */
    public function saveProjectTimeline(Request $request, $project_id)
    {
        $request['project_id'] = Crypt::decrypt($project_id);
        try {
            $timeline = $this->submit($request);

            if ($timeline) {
                $new_timeline = ProjectTimeline::find(Crypt::decrypt($timeline));
                $new_timeline->project_id = Crypt::decrypt($project_id);
                $new_timeline->save();
                $message = "The information has been saved.";
                $type = "success";
                alertNotify($message,null,$type);
            }
            else{
                $message = "Sorry! Something went wrong.";
                $type = "warning";
                alertNotify($message,null,$type);
            }
        }
        catch (\Exception $exception) {
            $message = $exception->getMessage();
            $type = "danger";
            alertNotify($message,null,$type);
        }
        return back();
    }

    /**
     * Update Project Timeline information
     * @param Request $request
     * @param $project_id
     * @return \Illuminate\Http\RedirectResponse
     */
    public function updateProjectTimeline(Request $request, $project_id)
    {
        $request['project_id'] = Crypt::decrypt($project_id);
        try {
            $timeline = $this->submit($request);

            if ($timeline) {
                $message = "The information has been updated.";
                $type = "success";
                alertNotify($message,null,$type);
            }
            else{
                $message = "Sorry! Something went wrong.";
                $type = "warning";
                alertNotify($message,null,$type);
            }
        }
        catch (\Exception $exception) {
            $message = $exception->getMessage();
            $type = "danger";
            alertNotify($message,null,$type);
        }
        return redirect()->route('project.projectTimeline',[$project_id]);
    }

    /**
     * Archive/Restore a Project
     * @param $project_id
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View
     * @throws \Throwable
     */
    public function archive($project_id)
    {
        $project = Project::find(Crypt::decrypt($project_id));

        if ($project->archived == 1) {
            $project->archived = false;
            $project->save();
            alertNotify('Project has been restored.',null,'success');
        }
        else {
            $project->archived = true;
            $project->save();
            alertNotify('Project has been archived.',null,'success');
        }

        return back();
    }

    /**
     * Delete a Project
     * @param $project_id
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View
     * @throws \Throwable
     */
    public function delete($project_id)
    {
        if (Project::destroy(Crypt::decrypt($project_id))) {
            alertNotify('Project has been deleted.',null,'success');
        }
        else {
            alertNotify('Sorry! Something went wrong. Try again.',null,'danger');
        }
        return back();
    }

    /**
     * Delete a Project Timeline
     * @param $project_timeline_id
     * @return \Illuminate\Http\RedirectResponse
     */
    public function deleteTimeline($project_timeline_id)
    {
        if (ProjectTimeline::destroy(Crypt::decrypt($project_timeline_id))) {
            alertNotify('Project Timeline has been deleted.',null,'success');
        }
        else {
            alertNotify('Sorry! Something went wrong. Try again.',null,'danger');
        }
        return back();
    }
}
