<?php

namespace App\Http\Controllers;

use App\Http\Helpers\FormRender;
use App\Http\Traits\FormSubmission;
use App\Models\Benefit;
use App\Models\Form;
use App\Models\Questionnaire;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Crypt;
use Illuminate\Support\Facades\DB;

class QuestionnaireController extends Controller
{
    use FormSubmission;

    public function __construct()
    {
        $this->middleware(['auth', 'verified']);
    }

    /**
     * Display a listing of the Questionnaires.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $forms = Form::activeQuestionnaires()->orderBy('title','asc')->get();

        return view('questionnaire.index',compact('forms'));
    }

    /**
     * Open the page for Assigning a Survey to a Benefit
     * @param $form_id
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View
     * @throws \Throwable
     */
    public function edit($form_id)
    {
        $form = Form::find(Crypt::decrypt($form_id));
        $get_render_form = new FormRender();
        $get_render_form->editable(0);
        $render_form = $get_render_form->renderFormView($form->table_name);

        $survey_benefit = null;
        $survey_status = null;
        if ($form->questionnaire) {
            $survey_benefit = $form->questionnaire->benefit->id;
            $survey_status = $form->questionnaire->status;
        }

        $benefit_types = Benefit::where('archived','=', 0)->orderBy('titles','asc')->get();

        return view('questionnaire.edit',compact('form','render_form','benefit_types',
            'survey_status','survey_benefit'));
    }

    /**
     * Save Assigned Survey to a Benefit
     * @param Request $request
     * @param $form_id
     * @return \Illuminate\Http\RedirectResponse
     */
    public function save(Request $request,$form_id)
    {
        $survey = Questionnaire::updateOrCreate([
            'form_id'   =>  Crypt::decrypt($form_id)
        ],[
            'benefit_id'    =>  $request->benefit_type,
            'status'    =>  $request->status,
        ]);
        if ($survey) {
            alertNotify('Successful! The Survey has been assigned to a Benefit.',null,'success');
        }
        else {
            alertNotify('Sorry! Something went wrong. Please try again',null,'warning');
            return back();
        }
        return redirect()->route('survey.list');
    }

    /**
     * Display Response for a particular Survey Form
     * @param $form_id
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View
     * @throws \Throwable
     */
    public function responses($form_id)
    {
        $form = Form::find(Crypt::decrypt($form_id));
        $benefit_id = Questionnaire::where('form_id','=',Crypt::decrypt($form_id))->where('status','=',1)->pluck('benefit_id')->first();

        $responses = DB::table($form->table_name)->paginate(10);
        $display_responses = new FormRender();
        $display_responses->displayAsDetails(true);

        return view('questionnaire.responses',compact('form','responses','display_responses','benefit_id'));
    }

    /**
     * Add a Response to a Survey
     * @param null $benefit_id
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View
     * @throws \Throwable
     */
    public function create($benefit_id=null,$response_id = null)
    {
        $render_form = $surveys = $survey = null;
        if ($benefit_id) {
            $survey = Questionnaire::find(Crypt::decrypt($benefit_id));
            $get_render_form = new FormRender();
            if ($response_id){
                $get_render_form->request_id($response_id);
            }
            $get_render_form->submitUrl(route('survey.saveResponse'));
            $render_form = $get_render_form->renderFormView($survey->form->table_name);
        }
        else {
            $surveys = Questionnaire::where('status','=', 1)->get();
        }

        return view('questionnaire.create',compact('surveys','survey','render_form'));
    }

    /**
     * Save Survey response
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function saveResponse(Request $request)
    {
//        dd($request->all());
        $benefit_id = $this->submit($request);

        if ($benefit_id) {
            $message = "The response has been saved.";
            $type = "success";
            alertNotify($message,null,$type);
        }
        if (isset($request->request_id)) {
            return redirect()->route('survey.responses',[cryptThis($request->form_id)]);
        }
        return back();
    }

    /**
     * Edit a Survey Response
     * @param $form_id
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View
     * @throws \Throwable
     */
    public function editResponse($form_id,$response_id)
    {
        $get_render_form = new FormRender();
        $get_render_form->request_id($response_id);
        $get_render_form->submitUrl(route('benefit.save'));

        $form = Form::find(Crypt::decrypt($form_id));
        $render_form = $get_render_form->renderFormView($form->table_name);

        $survey_benefit = null;
        $survey_status = null;
        if ($form->questionnaire) {
            $survey_benefit = $form->questionnaire->benefit->id;
            $survey_status = $form->questionnaire->status;
        }

        $benefit_types = Benefit::where('archived','=', 0)->orderBy('titles','asc')->get();

        return view('questionnaire.edit',compact('form','render_form','benefit_types',
            'survey_status','survey_benefit'));
    }

    /**
     * @param $id
     * @return \Illuminate\Http\RedirectResponse
     */
    public function destroyResponse(Request $request, $form_id)
    {
        $form = Form::find(Crypt::decrypt($form_id));
        if (DB::table($form->table_name)->where('id','=', Crypt::decrypt($request->response_id))->delete()){
            alertNotify('Response has been deleted successfully.',null, 'success');
        }
        else{
            alertNotify('Sorry! Something went wrong! Please try again.',null, 'danger');
        }
        return back();
    }
}
