<?php

namespace App\Http\Controllers;

use App\Http\Controllers\Controller;
use App\Http\Middleware\LoadUserPermissions;
use App\Models\Country;
use App\Models\Lookup;
use App\Models\LookupCode;
use App\Models\OfficeRank;
use App\Models\Permission;
use App\Models\Role;
use App\Models\Status;
use App\Models\ToastNotification;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\QueryException;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Crypt;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Session;
use Illuminate\Support\Str;
use Illuminate\Http\Request;
use App\Http\Requests\StoreRolesRequest;
use App\Http\Requests\UpdateRolesRequest;
use Illuminate\Support\Facades\Validator;
use Laratrust\Models\LaratrustRole;

class RolesController extends Controller
{
    public function __construct()
    {
        $this->middleware('auth');
    }

    public function index(Request $request){

        if (isset($request->role)) {
            $roles = Role::where('id',$request->role)->get();
        }
        else {
            if (Auth::user()->hasRole('webmaster')){
                $roles = Role::all();
            }else{
                $roles = Role::where('name','<>', 'webmaster')->where('instance_id','=', getActiveInstance())->orWhere('is_general','=',1)->get();
            }
        }

        $permissions = Permission::all();
        return view('roles.index', compact('roles','permissions'));
    }

    public function emptyForm(){
        if (Auth::user()->hasRole('webmaster')){
            $roles = Role::all();
        }else{
            $roles = Role::where('name','<>', 'webmaster')->where('instance_id','=', getActiveInstance())->orWhere('is_general','=',1)->get();
        }
        $role_permissions = array();
        $permissions = Permission::where('access_type','=','backend')->get(); //For static role-permissions
        $theView = view('roles.empty_form', compact('permissions','roles','role_permissions'));
        return $theView;
    }

    public function create(Request $request){

        $role_name = Str::slug(strtolower($request->role_name));

        $this->validate($request,[
            'role_name' => 'required|string|min:2|unique:roles,name',
            'code' => 'required|string|min:3|max:3|unique:roles,code',
            'lead' => 'nullable|integer',
            'access_type' => 'required|string|min:3',
            'role_desp' => 'nullable|string|min:3',
            'permissions' => 'required|array|min:1',
            'permissions.*' => 'required|numeric|min:1'
        ]);

        try{
            $owner = new Role();
            $owner->name         = $role_name;
            $owner->code    = strtoupper($request->code);
            $owner->display_name = $request->role_name;
            $owner->lead = $request->lead;
            $owner->access_type = $request->access_type;
            $owner->description  = $request->role_desp;
            $owner->instance_id  = getActiveInstance();
            $owner->save();

            $owner->attachPermissions($request->permissions);
            alertNotify(__('A new role has been added!'), null, 'success');
            $message = 'The Role has been added!';
            session()->flash('message', $message);
            session()->flash('status', 'success');

        }catch(\Illuminate\Database\QueryException $ex){
            $message = 'The Role may already exist or check enter appropriate data!';
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back();
        }

        return back();
    }

    public function edit(Request $request){
        $this->validate($request,[
            'name' => 'required|string|min:3'
        ]);
        $fixed_roles = config('user-management.default_roles');
        $role = Role::with(['permissions'])->where('name','=',$request->name)->first();
        $role_permissions = $role->permissions->pluck('id')->toArray();
        $roles = Role::where('instance_id','=', getActiveInstance())->orWhere('is_general','=',1)->get();

        $permissions = Permission::where('access_type','=','backend')->get();

        $theView = view('roles.edit', compact('role','roles','permissions','role_permissions','fixed_roles'))->render();
        return response()->json(['theView'=>$theView]);
    }

    public function update(Request $request)
    {
        if(in_array($request->id,config('settings.reserved-roles'))) {
            session()->flash('message', 'This role is not permitted for updates ');
            session()->flash('status', 'error');
            return back();
        }

        $role_name = Str::slug(strtolower($request->role_name));
        $fixed_roles = config('user-management.default_roles');
        $thisRole = Role::find($request->id);
        if (in_array($thisRole->name, $fixed_roles)){
            $role_name = $thisRole->name;
        }

        if (Role::where('name','=',$request->role_name)->where('id','<>',$request->id)->count() > 0){
            session()->flash('message', 'Role name already exists');
            session()->flash('status', 'error');
            return back();
        }

        if (Role::where('code','=',$request->role_name)->where('id','<>',$request->id)->count() > 0){
            session()->flash('message', 'Role code already exists');
            session()->flash('status', 'error');
            return back();
        }

        $this->validate($request,[
            'id' => 'required|numeric|min:1',
            'role_name' => 'required|string|min:3',
            'code' => 'required|string|min:3|max:3',
            'lead' => 'nullable|integer',
            'access_type' => 'required|string|min:3',
            'role_desp' => 'nullable|string|min:3',
            'permissions' => 'required|array|min:1',
            'permissions.*' => 'required|numeric|min:1'
        ]);

        try {
            $owner = Role::find($request->id);
            $owner->name = $role_name;
            $owner->code = strtoupper($request->code);
            $owner->display_name = $request->role_name;
            $owner->access_type = $request->access_type;
            $owner->lead = $request->lead;
            $owner->description = $request->role_desp;
            $owner->save();

            $owner->syncPermissions($request->permissions);
            $message = 'The Role has been updated!';
            session()->flash('message', $message);
            session()->flash('status', 'success');

            return back();
        }catch (QueryException $exception){
            $message = 'The Role may already exist or check enter appropriate data!'.$exception->getMessage();
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back();
        }
    }

    public function remove($id)
    {
        try {
            $role = Crypt::decrypt($id);
            Role::destroy($role);
            DB::table('role_user')->where('role_id','=', $role)->update(['role_id'=>0]);

            $message = 'Role has been removed!';
            session()->flash('message', $message);
            session()->flash('status', 'success');
            return back();
        }catch (QueryException $exception){
            $message = 'Something went wrong. Try again or contact Administrator.';
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back();
        }
    }

    public function loadPermissions(Request $request){
        $this->validate($request,[
            'accessType' => 'required|string|min:3',
        ]);

        $role_permissions = [];
        if (isset($request->role_id)) {
            $role = Role::with(['permissions'])->where('id','=',$request->role_id)->first();
            $role_permissions = $role->permissions->pluck('id')->toArray();
        }

        $permissions = Permission::where('access_type','=',$request->accessType)->get();

        $theView = view('roles.permissions', compact('permissions','role_permissions'))->render();
        return response()->json(['theView'=>$theView]);
    }

    public function activation($id)
    {
        try {
            $role = Role::find(Crypt::decrypt($id));
            if (!is_null($role)) {
                if ($role->status == 1){
                    $role->status = 0;
                    $message = 'The Role has been deactivated!';
                }
                else {
                    $role->status = 1;
                    $message = 'The Role has been activated!';
                }
                $role->save();

                session()->flash('message', $message);
                session()->flash('status', 'success');
            }
            else {
                $message = 'Role does not exist.';
                session()->flash('message', $message);
                session()->flash('status', 'error');
            }

            return back();
        }catch (QueryException $exception){
            $message = 'Something went wrong. Try again or contact Administrator.';
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back();
        }
    }

    public function hierarchy(){
        $statues_lookup_code = LookupCode::where('lookUpCode', 'STATUS')->first()?->id;
        $status = Lookup::where("lookup_code_id", $statues_lookup_code)->first();

        $roles = Role::query()->with('children')
            ->where('instance_id','=', getActiveInstance())
            ->orWhere('is_general','=',1)
            ->whereNull('lead')
            ->get();
        $countries = Country::all();

        $all_roles = Role::where('instance_id','=', getActiveInstance())->get();
        $office_classes = Lookup::where('lookup_code_id', 23)->orderBy('order', 'asc')->get();
        return view('roles.roles-hierarchy', compact('roles', 'countries', 'all_roles', 'office_classes'));
    }

    public function line_of_sight(Request $request)
    {
        $roles = Role::where('instance_id','=', getActiveInstance())
            ->orWhere('is_general','=',1)->get();


        if(isset($request->role_a) && isset($request->role_b)){
            // Retrieve the Role models for the selected roles
            $roleA = Role::find($request->role_a);
            $roleB = Role::find($request->role_b);

            // Get the hierarchy of roles from role_a to role_b (excluding roles higher than role_b)
            $rolesHierarchy = [];
            $currentRole = $roleA;

            while ($currentRole && $currentRole->id !== $roleB->id) {
                $rolesHierarchy[] = $currentRole;
                $currentRole = $currentRole->parent;
            }

            // Include role_b in the result
            $rolesHierarchy[] = $roleB;

        }else{
            $rolesHierarchy = [];
        }
        return view('roles.line-of-sight', compact('roles', 'rolesHierarchy'));
    }

    public function toggleStatus(Request $request)
    {
        $role = Role::find($request->role_id);
        if ($role->status == 1){
            $role->status = 0;
            $message = 'The Role has been deactivated!';
        }
        else {
            $role->status = 1;
            $message = 'The Role has been activated!';
        }

        session()->flash('message', $message);
        session()->flash('status', 'success');
        $role->save();

        return response()->json('success');

    }
}
