<?php


namespace App\Http\Controllers;
use App\Models\AccessLog;
use App\Models\Application;
use App\Models\Benefit;
use App\Models\BenefitPayment;
use App\Models\Client;
use App\Models\Country;
use App\Models\County;
use App\Models\Currency;
use App\Models\Deployment;
use App\Models\Instance;
use App\Models\InventoryItem;
use App\Models\ItemIssuance;
use App\Models\Lookup;
use App\Models\LookupCode;
use App\Models\MeasurementSystem;
use App\Models\Region;
use App\Models\Setting;
use App\Models\SubCounty;
use App\Models\User;
use App\Models\Village;
use Carbon\Carbon;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Http\Request;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Crypt;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;
use Illuminate\Validation\Rule;
use PHPUnit\Exception;
use QCod\AppSettings\SavesSettings;

class SettingController extends Controller
{
    use SavesSettings;

    public function __construct()
    {
        $this->middleware('auth');
    }

    public function countryData(Request $request)
    {
        $country = getCountry($request->id);
        if ($country) {
            return response()->json(['status'=>1,'data'=>$country]);
        }
        else {
            return response()->json(['status'=>0]);
        }
    }

    /**
     * List of Lookup Code
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View
     */
    public function lookupCode(Request $request){

        $codes = LookupCode::orderBy('lookupName','ASC');
        if (isset($request->option_type)) {
            $codes = $codes->where('id','=',$request->option_type);
        }
        $codes = $codes->get();
        return view('system_settings.lookup-code', compact('codes'));
    }

    /**
     * Create a new Lookup Code
     * @param Request $request
     * @return \Illuminate\Http\RedirectResponse
     * @throws \Illuminate\Validation\ValidationException
     */
    public function lookupCodeSave(Request $request){

        $this->validate($request,[
            'activeFlag' => 'required|boolean',
            'lookupCode' => 'nullable|string|min:1'
        ]);
        try{
            $createLookupCode = new LookupCode();
            $createLookupCode->lookupName  = Str::title($request->lookupName);
            $createLookupCode->lookupCode  = $request->lookupCode; // optional
            $createLookupCode->instance_id  = 0; // optional
            $createLookupCode->activeFlag  = $request->activeFlag; // optional
            $createLookupCode->created_by  = Auth::id(); // optional
            $createLookupCode->save();
            $message= 'Option Menu has been created!';
            session()->flash('message', $message);
            session()->flash('status', 'success');

        }catch(Exception $exception){
            $message = 'Sorry! Data did not save. Please try again. '.$exception->getMessage();
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back();
        }

        return back();
    }

    /**
     * Collect an Lookup Code data and post to frontend
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function lookupCodeEdit(Request $request)
    {
        $LookupCode = LookupCode::where('id','=',$request->id)->first();
        return response()->json($LookupCode);
    }

    /**
     * Update Lookup Code
     * @param Request $request
     * @return \Illuminate\Http\RedirectResponse
     * @throws \Illuminate\Validation\ValidationException
     */
    public function lookupCodeUpdate(Request $request)
    {
        $this->validate($request,[
            'id' => 'required|numeric|min:1',
            'activeFlag' => 'nullable|boolean',
            'lookupCode' => 'nullable|string|min:1'
        ]);

        try{
            $updateLookupCode = LookupCode::where('id','=',$request->id)->first();

            $updateLookupCode->lookupName  = Str::title($request->lookupName);
            $updateLookupCode->lookupCode  = $request->lookupCode; // optional
            $updateLookupCode->is_process  = $request->is_process; // optional
            $updateLookupCode->activeFlag  = $request->activeFlag; // optional
            $updateLookupCode->updated_by  = Auth::id(); // optional
            $updateLookupCode->save();

            $message= 'Option Menu has been updated!';
            session()->flash('message', $message);
            session()->flash('status', 'success');


        }catch(\Exception $exception){
            $message = 'Sorry! Data did not save. Please try again. '.$exception->getMessage();
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back();
        }
        return back();
    }

    /**
     * List of Options
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View
     */
    public function lookup(Request $request){

        $options = Lookup::orderBy('lookup_code_id','ASC');

        if (isset($request->option_type)) {
            $options = $options->where('lookup_code_id','=',$request->option_type);
        }
        if (isset($request->option_name)) {
            $options = $options->where('fullName','like','%'.$request->option_name."%");
        }
        $options = $options->get();

        $lookupCodes = LookupCode::orderBy('lookupName','ASC')->get();
        return view('system_settings.lookup', compact('options','lookupCodes'));
    }

    /**
     * List of Access Logs
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View
     */
    public function accessLog(Request $request){

        $logs = AccessLog::orderBy('updated_at','desc');
        $users = User::all();

        if (isset($request->user)) {
            $logs = $logs->where('user_id','=',$request->user);
        }
        $logs = $logs->paginate(50);

        return view('system_settings.access-log', compact('logs','users'));
    }

    public function forceLogout($user_id,$log)
    {
        $user = User::find(Crypt::decrypt($user_id));
        if ($user) {
            $user->force_logout = 1;
            $user->save();

            AccessLog::where('id','=',$log)->update(
                ['updated_at'  =>  Carbon::now()]
            );

            logUser('logout',Crypt::decrypt($user_id));

            $message= 'User has been logout';
            session()->flash('message', $message);
            session()->flash('status', 'success');
        }
        else {
            $message= 'Sorry! Try again';
            session()->flash('message', $message);
            session()->flash('status', 'error');
        }

        return back();
    }

    /**
     * Adding a new Option
     * @param Request $request
     * @return \Illuminate\Http\RedirectResponse
     * @throws \Illuminate\Validation\ValidationException
     */
    public function lookupSave(Request $request){

        $this->validate($request,[
            'fullName' => ['required','string','min:1',Rule::unique('lookups','fullName')->where(function ($query) use($request) {
                return $query->where('lookup_code_id', $request->lookup_code_id)->where('instance_id','=',0);
            })],
            'shortName' => ['required','string','min:1',Rule::unique('lookups','shortName')->where(function ($query) use($request) {
                return $query->where('lookup_code_id', $request->lookup_code_id);
            })],
            'color_code' => 'nullable|string',
            'order' => 'nullable|integer',
            'lookup_code_id' => 'required|integer',
            'activeFlag' => 'required|boolean',
        ]);
        try{
            $createLookup = new Lookup();
            $createLookup->fullName  = Str::title($request->fullName);
            $createLookup->shortName  = $request->shortName; // optional
            $createLookup->color_code  = $request->color_code; // optional
            $createLookup->instance_id  = 0;
            $createLookup->lookup_code_id  = $request->lookup_code_id;
            $createLookup->activeFlag  = $request->activeFlag; // optional
            $createLookup->order  = $request->order; // optional
            $createLookup->created_by  = Auth::id(); // optional
            $createLookup->save();

            $message = 'Option has been created!';
            session()->flash('message', $message);
            session()->flash('status', 'success');

        }catch(\Exception $exception){
            $message = 'Sorry! Option name already exist or check and enter appropriate data. Please try again. '.$exception->getMessage();
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back();
        }

        return back();
    }

    /**
     * Collect an Option data and post to frontend
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function lookupEdit(Request $request)
    {
        $option = Lookup::find($request->id);
        return response()->json($option);
    }

    /**
     * Update an Option
     * @param Request $request
     * @return \Illuminate\Http\RedirectResponse
     * @throws \Illuminate\Validation\ValidationException
     */
    public function lookupUpdate(Request $request)
    {
        $this->validate($request,[
            'id' => 'required|numeric|min:1',
            'fullName' => ['required','string','min:1',Rule::unique('lookups','lookup_code_id')->ignore($request->lookup_code_id)],
            'shortName' => ['required','string','min:1',Rule::unique('lookups','lookup_code_id')->ignore($request->lookup_code_id)],
            'color_code' => 'nullable|string',
            'integer' => 'nullable|integer',
            'lookup_code_id' => 'required|integer',
            'activeFlag' => 'nullable|boolean',
        ]);

        try{
            $updateLookup = Lookup::find($request->id);

            $updateLookup->fullName  = Str::title($request->fullName);
            $updateLookup->shortName  = $request->shortName; // optional
            $updateLookup->color_code  = $request->color_code; // optional
            $updateLookup->lookup_code_id  = $request->lookup_code_id;
            $updateLookup->activeFlag  = $request->activeFlag; // optional
            $updateLookup->order  = $request->order; // optional
            $updateLookup->updated_by  = Auth::id(); // optional
            $updateLookup->save();

            $message = 'Option has been updated!';
            session()->flash('message', $message);
            session()->flash('status', 'success');

        }catch(\Illuminate\Database\QueryException $ex){
            $message = 'Sorry! Option name already exist or check and enter appropriate data. Please try again. '.$ex->getMessage();
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back();
        }
        return back();
    }

    public function setConfig()
    {
        $countries = Country::all();
        $currencies = Currency::all();
        $clients = Client::orderBy('name', 'asc')->get();
        $action_type = 'Preferences';
        return view('settings.config', compact('action_type','countries','currencies', 'clients'));
    }

    public function saveConfig(Request $request)
    {
        $this->validate($request,[
            'app_logo' => 'mimes:png',
            'app_project_name'   =>    'required|string|min:3',
            'app_owner'   =>    'required|string|min:3',
            'app_email'   =>    'required|string|min:3',
            'app_main_currency'   =>    'required|integer|min:1',
            'app_contact'   =>    'required|string|min:10',
            'app_linkedIn_url'   =>    'nullable|string|min:10',
            'app_twitter_url'   =>    'nullable|string|min:10',
            'app_facebook_url'   =>    'nullable|string|min:10',
        ]);
        $app_background_image = $this->upload();
        $path = $this->upload('app_logo');

        if ($request->app_logo_clear != "clear") {
            if (isset($request->app_logo)) {

                settings()->set(['app_logo' => $path]);
            }
        }

        if ($request->remove_logo) {
            settings()->set(['app_logo' => null]);
        }

        // Handle the image upload
        if ($request->hasFile('app_logo')) {
            $image = $request->file('app_logo');
            $extension = $image->getClientOriginalExtension(); // Get the original file extension
            $newImgName = 'Alleviate.' . $extension; // Create the new file name

            // Move the uploaded file to the destination path with the new name
            $image->move(public_path('images/icons/'), $newImgName);
        }

        $deployment = Deployment::find(getActiveInstance());
        $deployment->name = $request->app_project_name;
        $deployment->save();



        $country = Country::find($request->app_country);
        $country_name = $call_code = null;

        if ($country) $country_name = $country->country_name; $call_code = $country->phone_code;
        $currency = Country::find($request->app_main_currency);
        if ($currency) {$currency_symbol = $currency->currency_code; $currency_code = $currency->currency_code;}
        else {$currency_symbol = env('DEFAULT_CURRENCY_SYMBOL','₵'); $currency_code = env('DEFAULT_CURRENCY_CODE','GHS');}

        settings()->set([
            'app_project_name' => $request->app_project_name,
            'app_email' => $request->app_email,
            'app_owner' => $request->app_owner,
            'app_contact' => $request->app_contact,
            'app_call_code' => $call_code,
            'app_main_currency' => $request->app_main_currency,
            'app_main_currency_symbol' => $currency_symbol,
            'app_main_currency_code' => $currency_code,
            'app_country' => $request->app_country,
            'app_country_name' => $country_name,
            'app_description' => $request->app_description,
            'app_linkedIn_url' => $request->app_linkedIn_url,
            'app_twitter_url' => $request->app_twitter_url,
            'app_facebook_url' => $request->app_facebook_url,
            'app_whatsApp_url' => $request->app_whatsApp_url,
            'app_copyright' => $request->app_copyright,
            'app_deployment_date' => $request->app_deployment_date,
            'app_address' => $request->app_address,
            'app_background_image' =>  $path
        ]);

        settings()->set(
            ['app_background_image' =>  $app_background_image]
        );

        $message = 'Update has been applied successfully';
        session()->flash('status', 'success');
        session()->flash('message', $message);
        return back();
    }

    public function saveTerms(Request $request)
    {
        $newData = Arr::except($request->all(), ['_token']);
        settings()->set($newData);
        $message = 'Terminology has been saved successfully';
        session()->flash('status', 'success');
        session()->flash('message', $message);
        return back();
    }

    /**
     * Function for upload image
     *
     * @param  string  $folder
     * @param  string  $key
     * @param  string  $validation
     *
     * @return false|string|null
     */
    public function upload($key = 'background_image', $validation = 'image|mimes:jpeg,png,jpg,gif,svg|max:2048|sometimes')
    {
        $folder = "settings";
        request()->validate([$key => $validation]);

        $file = null;
        if (request()->hasFile($key)) {
            $file = request()->file($key);
            $name = $file->hashName();
            $file = Storage::disk('public')->putFileAs($folder, request()->file($key),$name);
        }

        return 'storage/'.$file;
    }

    public function getCountryRegions($country_id){

        $country = Country::findOrFail($country_id);

        $regions = Region::where('country_id', $country->id)->orderBy('name', 'asc')->get();

        return response()->json($regions);

    }

    public function getCountryVillages($country_id)
    {
        $country = Country::findOrFail($country_id);
        if(env('COUNTY_ID') !== null){

            
            $villages = Village::where([['country_id', $country->id], ['county_id', env('COUNTY_ID')]])->orderBy('name', 'asc')->get();

        }else{
            
            $villages = Village::where('country_id', $country->id)->orderBy('name', 'asc')->get();
        }

        return response()->json($villages);
    }

    

    public function getRegionCounties($region_id){

        $region = Region::findOrFail($region_id);

        $counties = County::where('region_id', $region->id)->orderBy('county', 'asc')->get();

        return response()->json($counties);

    }

    public function getSubCounties($county_id){

        $county = County::findOrFail($county_id);

        $subcounties = SubCounty::where('county_id', $county->id)->orderBy('name', 'asc')->get();

        return response()->json($subcounties);

    }

    public function getVillages($subcounty_id){

        $subcounty = SubCounty::findOrFail($subcounty_id);

        //$subcounties = SubCounty::where('county_id', $county->id)->orderBy('name', 'asc')->get();
        $villages = Village::where('sub_county_id', $subcounty->id)->orderBy('name', 'asc')->get();

        return response()->json($villages);

    }

    public function get_inventory_items()
    {
        if(canDo('administrator','settings')){
            $inventory_items = InventoryItem::orderBy('item')->get();
        }else{
            $userDeploymentIds = auth()->user()->deployments->pluck('id');
            $inventory_items = InventoryItem::whereIn('deployment_id', $userDeploymentIds)->orderBy('id', 'desc')->get();
        }

        return response()->json(['inventory_items' => $inventory_items]);
    }

    public function get_application_items($application_id)
    {
        $application = Application::findOrFail($application_id);
        $items = ItemIssuance::where('application_id', $application->id)->get();
        return response()->json(['items' => $items]);
    }

    public function get_issuance_records($application_id)
    {
        $application = Application::findOrFail($application_id);
        $issuance_records = BenefitPayment::where('application_id', $application->id)->get();
        return response()->json(['issuance_records'=>$issuance_records]);
    }

    public function getIssuanceModeId($benefit_financial)
    {
        $issuance_mode_id = get_issuance_mode_id($benefit_financial);
        return response()->json(['issuance_mode_id' => $issuance_mode_id]);
    }

}
