<?php

namespace App\Http\Controllers;

use App\Http\Traits\EmailSender;
use App\Mail\UserCreated;
use App\Models\Role;
use App\Models\User;
use App\Models\UserProfile;
use Illuminate\Auth\Events\Registered;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Crypt;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Str;
use Illuminate\Validation\Rule;
use Yajra\DataTables\DataTables;

class UserManagementController extends Controller
{
    use EmailSender;
    public function __construct()
    {
        $this->middleware('auth');
    }

    /**
     * Users List
     *
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\View\View
     */
    public function users() {
//        $users = DB::table('users')->join('role_user','users.id','=', 'role_user.user_id')
//            ->join('roles','role_user.role_id','=', 'roles.id')
//            ->selectRaw('users.id as id,users.name as name,users.email as email,email_verified_at,users.created_at as created_at, roles.name as role')
//            ->orderBy('users.name','asc');
//        dd($users);
        return view('user-management.user.users');
    }

    /**
     * Get the list of active users vis DataTable
     * @return mixed
     * @throws \Exception
     */
    public function getBasicData()
    {
        $users = DB::table('users')->join('role_user','users.id','=', 'role_user.user_id')
            ->join('roles','role_user.role_id','=', 'roles.id')
//            ->whereNull('users.deleted_at')
            ->selectRaw('users.id as id,users.name as name,users.email as email,users.email_verified_at as email_verified_at,users.created_at as created_at, roles.display_name as role,users.deleted_at as deleted_at')
            ->orderBy('users.name','asc');

        return DataTables::of($users)
            ->addColumn('action', function ($user) {
                $activate = '<a href="'.route('user_management.deleteUser',[cryptThis($user->id)]).'"
                        onclick="return confirm(\'Are you sure you want to delete this user?\')" class="btn btn-sm mr-1 btn-icon btn-danger"
                        title="'. __("Delete User").'"><i class="la la-trash"></i></a>';
                $verify = "";
                if ($user->deleted_at != null) {
                    $activate = '<a href="'.route('user_management.activateUser',[cryptThis($user->id)]).'"
                        onclick="return confirm(\'Are you sure you want to restore this user?\')" class="btn btn-sm mr-1 btn-icon btn-success"
                        title="'. __("Restore User").'"><i class="la la-check"></i></a>';
                }
                if ($user->deleted_at == null && $user->email_verified_at == null) {
                    $verify = '<a href="'.route('user_management.verifyUser',[cryptThis($user->id)]).'"
                        onclick="return confirm(\'Are you sure you want to verify this user?\')" class="btn btn-sm mr-1 btn-icon btn-success"
                        title="'. __("Verify User").'"><i class="la la-check-double"></i></a>';
                }
                return '<a href="'.route('user_management.editUser',[cryptThis($user->id)]).'"
                class="btn btn-icon btn-sm mr-1 btn-light-primary" title="'. __("Edit User") .'"><i class="la la-edit"></i></a>'.$activate.$verify;
            })
            ->editColumn('email_verified_at', function ($user) {
                if ($user->email_verified_at) {
                    return '<span class="label label-success label-inline">'.__("Verified").'</span>';
                }
                else {
                    return '<span class="label label-light-info label-inline">'.__("Pending").'</span>';
                }
            })
            ->editColumn('created_at', function ($user) {
                return date('d M Y',strtotime($user->created_at));
            })
            ->filterColumn('created_at', function ($query, $keyword) {
                $query->whereRaw("DATE_FORMAT(created_at,'%d-%F-%Y') like ?", ["%$keyword%"]);
            })
            ->escapeColumns([])
            ->make();
    }

    /**
     * User create view
     *
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\View\View
     */
    public function createUser(){
        $roles = Role::orderBy('name', 'ASC')->get();

        return view('user-management.user.create-user', compact('roles'));
    }

    /**
     * Save new a user
     * @param Request $request
     * @return \Illuminate\Http\RedirectResponse
     * @throws \Illuminate\Validation\ValidationException
     */
    /**
     * @param Request $request
     * @return \Illuminate\Http\RedirectResponse
     */
    public function saveUser(Request $request)
    {
        $request->validate([
            'name' => 'required|string|max:255',
            'email' => 'required|string|email|max:255|unique:users'
        ]);
        try {
            $user = User::create([
                'name' => $request->name,
                'email' => $request->email,
                'is_admin' => $request->role,
                'last_instance_opened' => auth()->user()->last_instance_opened,
                //'password' => Hash::make($request->email),
                'password' => Hash::make('Password123'),
            ]);

            if ($user) {
                UserProfile::create([
                    'user_id' => $user->id
                ]);

                $user->roles()->sync($request->role);
            }

//            event(new Registered($user));
            //Send Email Verification
            $user->sendEmailVerificationNotification();

            //Send a Welcome email
            //Mail::to($request->email)->send(new UserCreated($user));

            alertNotify(__('The User has been created. An email has been sent to the user to verify account.'),
                __('New Account'),'success');
        }
        catch ( \Throwable $e ) {
            Log::error($e->getMessage() .' @ '.$e->getLine());
            DB::rollBack();

            alertNotify(__('An error was encountered. Please check the error logs for details information.'),
                __('User error!'),'danger');

            return back();

        }
        return back();
    }

    /**
     * Edit a user
     * @param $user_id
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View|\Illuminate\Http\RedirectResponse
     */
    public function editUser($user_id) {

        $user = User::find(Crypt::decrypt($user_id));
        $roles = Role::orderBy('name', 'ASC')->get();

        if (!empty($user)) {
            return view('user-management.user.edit-user', compact('user','roles'));
        } else {
            alertNotify('User does not exist',null, 'danger');
            return back() ;
        }
    }

    /**
     * Update User
     * @param Request $request
     * @return \Illuminate\Http\RedirectResponse
     */
    public function updateUser(Request $request) {

        $id = Crypt::decrypt($request->input('user'));

        $request->validate([
            'name' => 'required|string|max:255',
            'email' => [
                'required',
                'email',
                'max:255',
                Rule::unique('users')->ignore($id)
            ],
        ]);
        try {
            $user = User::find($id);
            $user->name = $request->name;
            $user->email = $request->email;
//            $user->is_admin = $request->role;

            if ($request->password) {
                $user->password = Hash::make($request->email);
            }
            if ($user) {
                $user->roles()->sync($request->role);
            }

//            if ($request->verify) {
//                $user->is_verified = true;
//            }else {
//                $user->is_verified = false;
//            }
            $user->save();

            alertNotify(__('The User has been updated.'), null,'success');
        }
        catch ( \Throwable $e ) {
            Log::error($e->getMessage() .' @ '.$e->getLine());
            DB::rollBack();

            alertNotify(__('An error was encountered. Please check the error logs for details information.'),
                __('User error!'),'danger');

            return back();

        }
        return redirect()->route('user_management.users');

    }

    /**
     * Soft Delete User
     * @param $user_id
     * @return \Illuminate\Http\RedirectResponse
     */
    public function deleteUser($user_id) {

        $user = User::destroy(Crypt::decrypt($user_id));

        if ($user) {
            alertNotify('User has been deleted.',null, 'success');
        } else {
            alertNotify('Something went wrong. Please try again',null, 'danger');
        }
        return back() ;
    }

    /**
     * Restore User
     * @param $user_id
     * @return \Illuminate\Http\RedirectResponse
     */
    public function activateUser($user_id) {

        $user = User::onlyTrashed()
            ->where('id', Crypt::decrypt($user_id))
            ->first();

        if ($user->restore()) {
            alertNotify('User has been restored.',null, 'success');
        }
       else {
            alertNotify('Something went wrong or user does not exist. Please try again',null, 'danger');
        }

        return back() ;
    }

    /**
     * Restore User
     * @param $user_id
     * @return \Illuminate\Http\RedirectResponse
     */
    public function verifyUser($user_id) {

        $user = User::where('id', Crypt::decrypt($user_id))
            ->first();
        if ($user) {
            $user->email_verified_at = date('Y-m-d H:i:s', strtotime(now()));

            if ($user->save()) {
                $emails = [$user->email];
                $message = 'Dear Sir/Madam,<br>';
                $message .= __("Your user account has been verified with the credentials below.");
                $message .= "<br><br>";
                $message .= "<p><em> Email: <strong>".$user->email."</strong></em></p>";
                $message .= "<p><em> Password: <strong>".$user->email."</strong></em></p>";
                $message .= "<br><br> Thank you.<br><br>Best Regards,";
                $subject = "Account Verified";

                $this->blast_mails($message,$subject,$emails);
                alertNotify('User has been verified.',null, 'success');
            }
            else {
                alertNotify('Something went wrong or user does not exist. Please try again',null, 'danger');
            }
        }
        else {
            alertNotify('User does not exist or deactivated. Please activate the account and try again',null, 'danger');
        }


        return back() ;
    }
}
