<?php

namespace App\Http\Controllers;

use App\Http\Middleware\LoadUserPermissions;
use App\Mail\AdminNewUserAccount;
use App\Mail\BeneficiaryInvitation;
use App\Models\AppPhoto;
use App\Models\Audit;
use App\Models\Country;
use App\Models\Deployment;
use App\Models\DeploymentInvitation;
use App\Models\GeneralNotification;
use App\Models\Lookup;
use App\Models\LookupCode;
use App\Models\Office;
use App\Models\Permission;
use App\Models\RecordTracker;
use App\Models\Role;
use App\Models\ToastNotification;
use App\Models\User;
use Carbon\Carbon;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Mail\NewUserAccount;
use App\Models\Applicant;
use App\Models\ApplicantDeployment;
use App\Models\City;
use App\Models\Client;
use App\Models\ClientBranding;
use App\Models\ContactDetail;
use App\Models\County;
use App\Models\HealthDetail;
use App\Models\IdentificationDetail;
use App\Models\MaritalDetail;
use App\Models\Region;
use App\Models\Village;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Crypt;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Http;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;
use PhpOffice\PhpSpreadsheet\IOFactory;

class UsersController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function __construct() {
        $this->middleware( ['auth:web'] );
    }

	public function users(Request $request)
    {
        $deployment = getActiveDeployment();
        $count_users_all = $deployment->users()->withTrashed()->whereNotIn('id',[1,2,3])->get();
        $count_users['users'] = $count_users_all->count();
        $count_users['approved_users'] = $count_users_all->where('account_verified_at','<>',null)->where('status','=',1)->count();
        $count_users['pending_users'] = $count_users_all->where('account_verified_at','=',null)->count();
        $count_users['expired_users'] = $count_users_all->where('deleted_at','<>',null)->count();

        $recent_users = $count_users_all->where('deleted_at','=',null)->sortByDesc('created_at')->take(5);

        $users = $deployment->users()->with(['roles','applications','office','applicant','_gender'])->orderBy('name');

        $roles = Role::orderBy('display_name')->get();
        $offices = Office::all();
        if (isset($request->value)) {
            $users = $deployment->users()->where('name','like',"%$request->value%")
                ->orWhere('firstname','like',"%$request->value%")
                ->orWhere('othername','like',"%$request->value%")
                ->orWhere('email','like',"%$request->value%");
        }
        if (isset($request->gender)) {
            $users = $users->where('gender','=',$request->gender);
        }
        if (isset($request->type)) {
            $role_users = DB::table('role_user')
                ->where('role_id','=',$request->type)
                ->pluck('user_id');
            $users = $users->whereIn('id',$role_users);
        }

        $users = $users->paginate(100);
        return view('user_management.users', compact('users', 'roles','offices','count_users','recent_users'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function addAccount()
    {
        // if (!Auth::user()->hasRole('administrator')) return redirect()->route('account.profile');
        $user = null;
        $action_type = 'Create';
        $self = false;
        $user_types = getUserTypes();
        $counties = getInstanceCounties();
        return view('user_management.profile', compact('user','counties','action_type','user_types','self'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\RedirectResponse
     */
    public function store(Request $request)
    {
    	$request->validate([
		    'name' => 'required|string|max:255',
		    'email' => 'required|string|email|max:255|unique:users',
		    'password' => 'required|string|min:6|confirmed',
	    ]);

    	$user = User::create([
		    'name' => $request->get('name'),
		    'email' => $request->get('email'),
		    'office_id' => $request->get('office_id'),
		    'password' => Hash::make($request->get('password')),
            'password_changed_at' => Carbon::now()->toDateTimeString(),
		    'role_id' => $request->get('role')
	    ]);

    	$user->syncRoles([$request->get('role')]);

    	notify(new ToastNotification('Successful', $user->name.' has been added as a user of the system', 'success'));
    	return redirect()->route('users.index');

    }

    /**
     * Display the specified resource.
     *
     * @param  \App\Models\User  $user
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View
     */
    public function show(User $user)
    {

        $user_roles = implode(', ',$user->roles->pluck('display_name')->toArray());
        $action_type = "Edit";
        $user_types = Role::orderBy('display_name')->get();

        $activity_logs = Audit::query()
            ->where('user_id', '=', $user->id)
            ->orderBy('created_at', 'desc')
            ->limit(5)
            ->get();

        return view('user_management.profile', compact('user', 'user_roles','activity_logs','action_type','user_types'));

    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Models\User  $user
     * @return \Illuminate\Http\RedirectResponse
     */
    public function update(Request $request, User $user)
    {
	    $request->validate([
		    'name' => 'required|string|max:255',
	    ]);


	    $user->name = $request->get('name');
	    $user->role_id = $request->get('role');
	    $user->office_id = $request->get('office_id');

	    $user->save();

	    $user->syncRoles([$request->get('role')]);

	    notify(new ToastNotification('Success!', 'Changes have been saved successfully!', 'success'));
	    return redirect()->back();

    }

    /**
     * Reset Password Random
     * @param $user_id
     *
     * @return \Illuminate\Http\RedirectResponse
     */
    public function resetPasswordRandom($user_id) {
        $user = User::find(Crypt::decrypt($user_id));
        $password = Str::random(10);
        if ($user) {
            $user->password = Hash::make($password);
            if ($user->save()) {
                $message = "New password is $password.";
                session()->flash('status', 'success');
                $sms = setAppDefault('app_project_name').": This is your new password: $password to access your account. Thank you.";

                if(!is_null($user->phone)) sendSMS($user->phone,$sms);
            } else {
                $message = "Something went wrong. Please try again";
                session()->flash('status', 'error');
            }
        } else {
            alertNotify('Account does not exist in our system.',null, 'danger');
            $message = "New password is $password";
            session()->flash('status', 'error');
        }
        session()->flash('message', $message);
        return back() ;
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Models\User  $user
     * @return \Illuminate\Http\JsonResponse
     */
    public function deleteAccount($user)
    {
	    try {
	        if(User::destroy(Crypt::decrypt($user))){
                $message = 'User has been deleted successfully';
                session()->flash('status', 'success');
            }else{
                $message = 'User has been deleted successfully';
                session()->flash('status', 'error');
            }
            session()->flash('message', $message);
		    return back();

	    } catch ( \Exception $e ) {
            $message = 'Sorry!, Something went wrong. Try again';
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back();
	    }
    }

    public function activation(Request $request)
    {
        $user = User::find($request->id);
        if ($user) {
            if ($user->deactivated_at == null) {
                $user->deactivated_at = Carbon::now();
                $user->status = 0;
                $message = 'User Account has been deactivated';
            }
            else {
                $user->deactivated_at = null;
                $user->status = 1;
                $message = 'User Account has been activated';
            }

            if ($user->save()) {
                $message_status = 'success';
            }
            else {
                $message = 'Sorry something went wrong. Please try again.';
                $message_status = 'danger';
            }
        }
        $alert = '<div role="alert" class="alert alert-'.$message_status.'">'.$message.'</div>';
        return response()->json(['message'=>$alert]);
    }

    /**
     * Show the profile page.
     *
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View
     */
    public function profileold()
    {
        $user = Auth::user();

        $user_roles = implode(', ',$user->roles->pluck('display_name')->toArray());


        $activity_logs = Audit::query()
                    ->where('user_id', '=', Auth::user()->id)
                    ->orderBy('created_at', 'desc')
                    ->limit(5)
                    ->get();

        return view('user_management.profile', compact('user', 'user_roles','activity_logs'));
    }

    public function profile()
    {
        $activity_logs = Audit::query()
            ->where('user_id', '=', Auth::user()->id)
            ->orderBy('created_at', 'desc')
            ->limit(5)
            ->get();
        $user = Auth::user();
        $counties = County::where([['county_id', 115], ['status', 1]])->get();

        return view('user_management.user-profile', compact('user','activity_logs','counties'));
    }

    public function userProfile($userId)
    {
        $activity_logs = Audit::query()
            ->where('user_id', '=', Auth::user()->id)
            ->orderBy('created_at', 'desc')
            ->limit(5)
            ->get();
        $user = User::find(Crypt::decrypt($userId));
        return view('user_management.user-profile', compact('user','activity_logs'));
    }

    /**
     * Show the edit profile page.
     *
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View
     */
    public function editAccount($user_id=null)
    {
        if (!is_null($user_id)) {
            $user = User::find(Crypt::decrypt($user_id));
            $self = false;
        }
        else {
            $user = Auth::user();
            $self = true;
        }

        $action_type = 'Edit';
        $user_types = getUserTypes();
        $invitations = auth()->user()->invitations()->get();

        return view('user_management.profile',
            compact('user','action_type','user_types','invitations','self'));
    }

    public function activateInvitation(Request $request)
    {
        $invitation = DeploymentInvitation::find(Crypt::decrypt($request->invitation));

        $redirect = false;
        $deployment = null;

        if ($invitation) {
            if ($invitation->accepted) {
                $invitation->accepted = false;
                auth()->user()->deployments()->detach($invitation->deployment_id);
                DB::table('role_user')->where('role_id','=',$invitation->role_id)
                    ->where('user_id','=',auth()->id())
                    ->where('instance_id','=',$invitation->deployment_id)
                    ->delete();
                $message = 'Invitation has been cancelled successfully.';
                $status = 'success';

                if (getActiveInstance() == $invitation->deployment_id) {
                    $redirect = true;
                    $deployment = auth()->user()->deployments()->first();
                }
            }
            else {
                $invitation->accepted = true;
                auth()->user()->deployments()->syncWithoutDetaching($invitation->deployment_id);
                auth()->user()->roles()->syncWithoutDetaching([$invitation->role_id => ['instance_id' => $invitation->deployment_id]]);

                updateNotification($invitation->id,$invitation->getTable());

                $message = 'You have accepted to collaborate successfully.';
                $status = 'success';
            }

            $invitation->save();
        }
        else {
            $message = 'Invalid invitation';
            $status = 'error';
        }

        notify($message,$status,'Invitation');

        if ($redirect) {
            if ($deployment) {
                return redirect()->route('subscription.switch',[$deployment]);
            }
            else {
                Auth::logout();
            }
        }

        return back();
    }

    /**
     * Saves the profile new data.
     *
     * @return \Illuminate\Http\RedirectResponse
     */
    public function save_profile(Request $request)
    {
        
        $this->validate($request,[
            'name' => 'required|string|min:2',
            'firstname' => 'required|string|min:2',
            'phone' => 'required|string|min:9',
            'gender' => 'required|numeric|min:0',
            'othername' => 'nullable|string|min:2',
            'email' => 'nullable|string|min:2',
            'country_id' => 'nullable|string|min:2',
            'type' => 'nullable|numeric|min:1',
            'profile_photo' => 'nullable|file'
        ]);

        if(isset($request->country)){
            $selected_country = Country::find($request->country);
        }

        try {
            if (isset($request->user_id)) {

                if (isset($request->phone)) {
                    $existing_phone = User::where('phone', '=',cleanString($request->phone))
                        ->where('call_code', '=',$request->call_code)
                        ->where('id', '<>',Crypt::decrypt($request->user_id))
                        ->count();
                    if ($existing_phone > 0) {
                        session()->flash('status', 'error');
                        session()->flash('message', 'This phone number has already been used.');
                        return back()->withInput();
                    }
                }

                if (isset($request->email)) {
                    $existing_email = User::where('email', '=',$request->email)
                        ->where('id', '<>',Crypt::decrypt($request->user_id))
                        ->count();
                    if ($existing_email > 0) {
                        session()->flash('status', 'error');
                        session()->flash('message', 'This email already exists.');
                        return back()->withInput();
                    }
                }

                $newUser = false;
                $profile = User::find(Crypt::decrypt($request->user_id));
                $profile->updated_by = Auth::id();
            }
            else {

                

                if (isset($request->phone)) {
                    $existing_phone = User::where('phone', cleanString($request->phone))
                        ->where('call_code', $request->call_code)
                        ->count();
                    if ($existing_phone > 0) {
                        session()->flash('status', 'error');
                        session()->flash('message', 'This phone number already exists');
                        return back()->withInput();
                    }
                }
                if (isset($request->email)) {
                    $existing_email = User::where('email', '=',$request->email)->count();
                    if ($existing_email > 0) {
                        session()->flash('status', 'error');
                        session()->flash('message', 'This email already exists.');
                        return back()->withInput();
                    }
                }

                $newUser = true;
                $profile = new User();
                $profile->last_instance_opened = auth()->user()->last_instance_opened ?? null;
                $profile->created_by = Auth::id();
                $profile->password = Hash::make('Password123');
                $profile->job_title = $request->job_title;
            }

            $profile->name = $request->name;
            $profile->firstname = $request->firstname;
            $profile->othername = $request->othername;
            if (!is_null($request->dob)) {
                $profile->dob = date('Y-m-d', strtotime($request->dob));
            }
//            dd($request->all());

            $profile->email = $request->email;
            $profile->username = $request->username;
            $profile->country_id = $request->country;
            $profile->call_code = $request->call_code;
            $profile->office_id = $request->office_id;
            $profile->job_title = $request->job_title;
            $profile->supervisor = $request->supervisor ?? null;
            $profile->call_code = '+'.$selected_country->phone_code;
            $profile->phone = cleanString($request->phone);
            $profile->gender = $request->gender;
            $profile->account_verified_at = date('Y-m-d H:i:s', strtotime(now()));
            $profile->is_admin = (in_array($request->type, [config('settings.admin_role_id'),env('OWNER_ROLE')]))?1:0;
            $profile->save();

            if ($profile->save()) {
                $profile->roles()->sync([$request->type => ['instance_id' => getActiveInstance()]]);

                if ($request->type != config('settings.beneficiary_role_id')) {
                    $profile->deployments()->syncWithoutDetaching([getActiveInstance()]);
                }
            }

            if($request->file('profile_photo')) {
                $url = $this->uploadProfile($profile);

                if (isset($request->user_id)) {
                    AppPhoto::where('user_id','=',$profile->id)->update([
                        'status'        =>      0,
                        'updated_by'   =>      Auth::id(),
                    ]);
                }

                AppPhoto::create([
                    'user_id'       =>      $profile->id,
                    'type'          =>      'Profile',
                    'scan'          =>      $url,
                    'url'            =>      $url,
                    'status'        =>      1,
                    'created_by'   =>      Auth::id(),
                ]);
            }

            $message = 'Personal Information has been saved successfully';
            session()->flash('status', 'success');
            if (isset($request->user_id)) {
                $message = 'Personal Information has been updated successfully';
            }
            session()->flash('message', $message);

            $code = null;
            $deployment = Deployment::findOrFail(getActiveInstance());
            if ($request->type == config('settings.beneficiary_role_id') && $newUser) {
                $code = strtoupper(Str::random(4));
                $profile->verification_code =  $code;
                $profile->save();
                $sms = "You have been invited by ".$deployment->name." to join ".setAppDefault('app_project_name').". Use this verification code - $code to activate your account after login. Thank you.";

                if (!is_null($request->phone)) sendSMS(phoneNumberFormat($request->phone,$request->call_code),$sms);
            }

            if($profile->email != null && $newUser){
                $role = Role::findOrFail($request->type);
                if ($request->type == config('settings.beneficiary_role_id')) {
                    Mail::to([$profile->email])->send(new BeneficiaryInvitation($profile,$deployment,$role->display_name,$code));
                }
                else {
                    Mail::to([$profile->email])->send(new AdminNewUserAccount($profile,$deployment,$role->display_name));
                }
            }
            return redirect()->route('account.users');
        }
        catch (\Exception $exception) {
            $message = 'Sorry something went wrong. Please try again. '.$exception->getMessage();
            session()->flash('message', $message);
            session()->flash('status', 'error');
            return back()->withInput();
        }
    }

    public function uploadProfile($profile,$key = 'profile_photo', $validation = 'image|mimes:jpeg,png,jpg,gif,svg|max:2048|sometimes')
    {
        request()->validate([$key => $validation]);

        $file = null;
        if (request()->hasFile($key)) {
            $location = 'users/'.$profile->id.'/profile';
            $file = request()->file($key);
            $name = $file->hashName();
            $file = Storage::disk('public')->putFileAs($location, request()->file($key),$name);
            $file = 'storage/'.$file;
        }

        return $file;
    }

    /**
     * Show the change the password page.
     *
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\View\Factory|\Illuminate\Contracts\View\View
     */
    public function password()
    {
        return view('user_management.user-password');
    }

    /**
     * Saves the new password.
     *
     * @return \Illuminate\Http\RedirectResponse
     */
    public function changePassword(Request $request)
    {
        $request->validate([
            'user_id' => ['required'],
            'password' => ['required', 'confirmed']
        ]);

        $user = User::find(Crypt::decrypt($request->user_id));
        if (!$user) {
            $message = 'User account cannot be accessed. Please try again.';
            session()->flash('status', 'error');
            session()->flash('message', $message);
            return back();
        }
        if (!Auth::user()->hasRole('administrator')){
            if (!Hash::check($request->current_password, $user->password)) {
                $message = 'Your current password is incorrect. Please enter the right password to proceed.';
                session()->flash('status', 'error');
                session()->flash('message', $message);
                return back();
            }
        }

        $user->password = Hash::make($request->password);
        $user->remember_token = Str::random(60);

        if ($user->save()) {
            $message = 'Password has been changed successfully.';
            session()->flash('status', 'success');
        }
        else {
            $message = 'Password failed to change. Please try again.';
            session()->flash('status', 'error');
        }
        session()->flash('message', $message);
        return back();
    }

    public function update_user_profile_picture(Request $request) {
        $this->validate($request, [
            'profile_picture' => 'nullable|image',
        ]);

        //find inmate record
        $updateUserProfileImage = User::find($request->user_id);;

        $profile_picture_new_name = null;

        if($request->profile_picture != null) {
            $profile_picture = $request->profile_picture;
            $profile_picture_new_name = time().$profile_picture->getClientOriginalName();
            $profile_picture->move(public_path('uploads/profile_pictures'), $profile_picture_new_name);
            $updateUserProfileImage->profile_photo_path = $profile_picture_new_name;
        }

        //save data
        $updateUserProfileImage->save();

        $request->session()->flash('success','User details updated successfully');
        return redirect()->route('user.profile');
    }

    public function memberRecords()
    {
        $records = RecordTracker::all();
        $records->load('requester','applicant');
        return view('user_management.member-records-update',compact('records'));
    }

    public function supportTickets()
    {
        return view('user_management.support-ticket');
    }

    public function paySlip()
    {
        return view('user_management.payslip');
    }

    public function paymentBatch()
    {
        return view('user_management.payment-batch');
    }

    public function hierarchy(){
        $users = User::with('children')
            ->whereHas('roles',function ($query){
                $query->where('name','!=','beneficiary');
            })
            ->whereNull('supervisor')
            ->get();
        $countries = Country::all();

        $all_users = User::all();
        $office_classes = Lookup::where('lookup_code_id', 23)->orderBy('order', 'asc')->get();
        return view('user_management.user-hierarchy', compact('users', 'countries', 'all_users', 'office_classes'));
    }

    public function user_branding(){
        $countries = Country::all();
        $clients = Client::orderBy('name', 'asc')->get();
        return view('branding-settings', compact('countries', 'clients'));
    }

    public function set_user_branding(Request $request, $user_id)
    {
        //dd($request);
        //validate user input
        $request->validate([
            'image' => 'image|mimes:jpeg,png,jpg,gif,svg|max:10048',
            'app_project_name'=> 'required',
            'app_owner'=> 'required',
            'app_email'=> 'required',
            'app_country'=> 'required',
            'app_main_currency'=> 'required',
            'app_contact'=> 'required',
            'app_copyright'=> 'max:255',
            
        ]);

        

        $client_branding = ClientBranding::where('client_id', auth()->user()->hasOrganisation()->id)->first();

        if(!$client_branding){

            $client_branding = ClientBranding::create([
                'client_id'=>auth()->user()->hasOrganisation()->id,
                'app_project_name' => $request->app_project_name,
                'app_description'=> $request->app_description,
                'app_email' => $request->app_email,
                'app_country'=> $request->app_country,
                'app_copyright'=> $request->app_copyright,
                'app_main_currency'=> $request->app_main_currency,
                'app_contact'=> $request->app_contact,
                'app_address'=> $request->app_address,
                'app_twitter_url' => $request->app_twitter_url,
                'app_linkedin_url' => $request->app_linkedIn_url,
                'app_facebook_url' => $request->app_facebook_url,
            ]);

        }else{
            $client_branding->app_project_name = $request->app_project_name;
            $client_branding->app_description = $request->app_description;
            $client_branding->app_email = $request->app_email;
            $client_branding->app_country = $request->app_country;
            $client_branding->app_copyright = $request->app_copyright;
            $client_branding->app_address = $request->app_address;
            $client_branding->app_main_currency = $request->app_main_currency;
            $client_branding->app_contact = $request->app_contact;
            $client_branding->app_linkedin_url = $request->app_linkedIn_url;
            $client_branding->app_facebook_url = $request->app_facebook_url;
            $client_branding->app_twitter_url = $request->app_twitter_url;
            $client_branding->app_contact = $request->app_contact;

            $client_branding->save();
        }

        $deployment = Deployment::find(getActiveInstance());
        $deployment->name = $request->app_project_name;
        $deployment->description = $request->app_description;
        $deployment->email = $request->app_description;
        $deployment->address = $request->app_address;
        $deployment->contact = $request->app_contact;
        $deployment->country_id = $request->app_country;
        $deployment->country_name = getCountry($request->app_country,'name');
        $deployment->currency_code = getCountry($request->app_country,'currency_code');

        //$deployment->linkedin_url = $request->app_linkedIn_url;
        //$deployment->twitter_url = $request->app_twitter_url;
        //$deployment->facebook_url = $request->app_facebook_url;
        $deployment->save();


        // Handle the image upload
        if ($request->hasFile('image')) {
            
            $image = $request->image;
            $newImgName = time().'-'.$image->getClientOriginalName();
            $request->image->move(public_path('app-uploads/clients'), $newImgName);

            $client_branding->logo = $newImgName;
            $client_branding->save();
 
        }

        return back()->with('success', 'Success');

       
    }

    public function get_cord()
    {
        $user_country_id = auth()->user()->hasOrganisation()?->country?->id ?? 82;

        $country = Country::findOrFail();
        // Fetch data from REST Countries API
        $response = Http::get("https://restcountries.com/v3.1/name/{$user_country}");

        if ($response->successful()) {
            $data = $response->json();
            if (!empty($data)) {
                // Extract the latitude and longitude from the first matching country
                $country = $data[0];
                $coordinates = [
                    'latitude' => $country['latlng'][0],
                    'longitude' => $country['latlng'][1],
                ];

                Country::where('country_name', $user_country)->first();

                // Pass the coordinates to the view
                return view('country.coordinates', compact('coordinates', 'name'));
            }
        }

        dd('Unsuccessful');
    }

    public function get_cord2()
    {
        $countries = Country::whereNull('latitude')->orWhereNull('longitude')->get();

        foreach ($countries as $country) {
            //$this->info("Fetching coordinates for {$country->country_name}...");

            $response = Http::get("https://restcountries.com/v3.1/alpha/{$country->country_code}");

            if ($response->successful()) {
                $data = $response->json();
                if (!empty($data)) {
                    $countryData = $data[0];
                    $country->latitude = $countryData['latlng'][0];
                    $country->longitude = $countryData['latlng'][1];
                    $country->save();

                    //$this->info("Updated {$country->country_name} with latitude: {$country->latitude} and longitude: {$country->longitude}");
                } else {
                    //$this->warn("No data found for {$country->country_name}");
                }
            } else {
                //$this->error("Failed to fetch data for {$country->country_name}");
            }
        }

        dd('Done');

    }

    public function import_beneficiaries(){
        $users = User::whereHas('deployments', function ($query) {
            $query->whereHas('admin');
        })->orderBy('firstname', 'asc')->get();
        $deployments = Deployment::orderBy('name', 'asc')->get();
        return view('beneficiaries.import', compact('users', 'deployments'));
    }

    public function get_users_deployments($user_id)
    {
        $user = User::findOrFail($user_id);
        return response()->json($user->deployments);

    }

    public function handle_beneficiaries_import2(Request $request)
    {
        ini_set('max_execution_time', 300); 

        $request->validate([
            'deployment'=>'required',
            'file' => 'required|mimes:xlsx,xls,csv'
        ]);

        $deployment = Deployment::findOrFail($request->deployment);
        $deploymentCountry = Country::findOrFail($deployment->country_id);

        // Load the uploaded file
        $file = $request->file('file');
        $spreadsheet = IOFactory::load($file->getPathname());
        $worksheet = $spreadsheet->getActiveSheet();


        $rowIndex = 2;
        $omitted_rows = [];

        while ($worksheet->getCell('A' . $rowIndex)->getValue() !== null) {
            // Extract data from each column in the current row

            $surname =  $worksheet->getCell('AX'. $rowIndex)->getValue() !== 'NULL' ? ucwords($worksheet->getCell('AX'. $rowIndex)->getValue())  : null;
            $firstname = $worksheet->getCell('AV'. $rowIndex)->getValue() !== 'NULL' ? ucwords($worksheet->getCell('AV'. $rowIndex)->getValue())  : null;
            $other_names =  $worksheet->getCell('AW'. $rowIndex)->getValue() !== 'NULL' ? ucwords($worksheet->getCell('AW'. $rowIndex)->getValue())  : null;
            $genderChar = $worksheet->getCell('BC'. $rowIndex)->getValue();
            $gender = $genderChar == 'M' ? 'Male' : 'Female';
            $dobSerial = $worksheet->getCell('BD'. $rowIndex)->getValue();
            if (is_numeric($dobSerial)) {
                $dob = Carbon::createFromDate(1900, 1, 1)->addDays($dobSerial - 2)->format('Y-m-d');
            } else {
                $dob = Carbon::parse($dobSerial)->format('Y-m-d');
            }

            $place_of_birth = 'Kenya';

            $state_of_birth = env('COUNTY_NAME');
            
            
            $state_id = $this->get_region_id(env('COUNTY_NAME'));

            $city_id = $this->get_county_id(env('COUNTY_NAME'));

            $father_alive = $worksheet->getCell('BG'. $rowIndex)->getValue() == 'No' ? 0 : 1;
            $mother_alive = $worksheet->getCell('BH'. $rowIndex)->getValue() == 'No' ? 0 : 1;
            $hse_condition = $worksheet->getCell('AM'. $rowIndex)->getValue();
            $pmt = $worksheet->getCell('BQ'. $rowIndex)->getValue();
            $learning_stat = ucwords($worksheet->getCell('BM'. $rowIndex));

            //Get village/ward id
            $village= Village::where('name', ucwords($worksheet->getCell('G'. $rowIndex)))->first();


            if(empty($village)){
                array_push($omitted_rows, $rowIndex);
            }else{

                $applicant = Applicant::create([
                    'regID'      => 'prelimID',
                    'surnames'    => $surname,
                    'first_names'  => $firstname,
                    'other_names' => $other_names,
                    'dobs'        => $dob,
                    'genders'     => $gender,
                    'country_id'     => $deployment->country_id,
                    'city_id'     => $city_id,
                    'state_id'     => $state_id ,
                    'place_of_births' => $place_of_birth,
                    'state_of_births' => $state_of_birth,
                    'country_of_births' => $deploymentCountry->country_name,
                    'nationalities'=>$deploymentCountry->nationality,
                    'father_alive'=> $father_alive,
                    'mother_alive'=> $mother_alive,
                    'household_condition' => ucwords($hse_condition),
                    'pmtScore' => $pmt,
                    'education_level' => ucwords($worksheet->getCell('BN'. $rowIndex)),
                    'learning_status' => ucwords($learning_stat),
                    'occupation' => null,
                    'added_by'=>auth()->user()->id,
                    'village_id'=>$village->id
                ]);

                $applicant->regID = $this->generateRefID($request) . '-' . $applicant->id;
                $applicant->save();

                ApplicantDeployment::create([
                    'applicant_id'=>$applicant->id,
                    'deployment_id'=>$request->deployment
                ]);

                ContactDetail::create([
                    'type_name'=> 'Mobile Number',
                    'type'=> 109,
                    'mobile'=> '+254'.$worksheet->getCell('BA'. $rowIndex)->getValue(),
                    'applicant_number'=>$applicant->regID,
                    'addedBy'=>auth()->user()->id,
                    'status'=>'Verified'
                ]);

                ContactDetail::create([
                    'type_name'=> 'Residential Address',
                    'type'=> 107,
                    'applicant_number'=>$applicant->regID,
                    'country'=> 115,
                    'region'=>$this->get_region_id(env('COUNTY_NAME')),
                    'city'=>$this->get_county_id(env('COUNTY_NAME')),
                    'village'=>$village->name,
                    'landmark' => $worksheet->getCell('H'. $rowIndex)->getValue(),
                    'addedBy'=>auth()->user()->id,
                    'status'=>'Verified'
                ]);

                if($worksheet->getCell('AY'. $rowIndex) !== 'NONE' && $worksheet->getCell('AY'. $rowIndex) !== 'NULL' && $worksheet->getCell('AZ'. $rowIndex) !== 'NULL'){
                    IdentificationDetail::create([
                        'applicant_number'=>$applicant->regID,
                        'id_type' => ucwords($worksheet->getCell('AY'. $rowIndex)),
                        'id_number' => $worksheet->getCell('AZ'. $rowIndex),
                        'addedBy'=>auth()->user()->id,
                    ]);


                }

                if($worksheet->getCell('BE' . $rowIndex)->getValue() !== 'None' && $worksheet->getCell('BE' . $rowIndex)->getValue() !== 'NULL'){

                    if($worksheet->getCell('BE' . $rowIndex)->getValue() == 'Never Married'){
                        MaritalDetail::create([
                            'applicant_number' => $applicant->regID,
                            'marriage_status' => 'Single',
                            'addedBy' => auth()->user()->id,
                            'status'=>'New'
                        ]);
                    }

                    elseif($worksheet->getCell('BE' . $rowIndex)->getValue() == 'Married Monogamous'){

                        MaritalDetail::create([
                            'applicant_number' => $applicant->regID,
                            'marriage_status' => 'Married Monogamous',
                            'spouse_name' => 'Unknown',
                            'addedBy' => auth()->user()->id,
                            'status'=>'New'
                        ]);

                    }

                    elseif($worksheet->getCell('BE' . $rowIndex)->getValue() == 'Married Polygamous'){

                        MaritalDetail::create([
                            'applicant_number' => $applicant->regID,
                            'marriage_status' => 'Married Polygamous',
                            'spouse_name' => 'Unknown',
                            'addedBy' => auth()->user()->id,
                            'status'=>'New'
                        ]);
                        
                    }

                    elseif($worksheet->getCell('BE' . $rowIndex)->getValue() == 'Widowed'){

                        MaritalDetail::create([
                            'applicant_number' => $applicant->regID,
                            'marriage_status' => 'Ended By Death',
                            'spouse_name' => 'Unknown',
                            'addedBy' => auth()->user()->id,
                            'status'=>'New'
                        ]);
                        
                    }

                    elseif($worksheet->getCell('BE' . $rowIndex)->getValue() == 'Divorced or Separated'){

                        MaritalDetail::create([
                            'applicant_number' => $applicant->regID,
                            'marriage_status' => 'Separated',
                            'spouse_name' => 'Unknown',
                            'addedBy' => auth()->user()->id,
                            'status'=>'New'
                        ]);
                        
                    }

                }

                if($applicant->healthDetails()->count() < 1){

                    if($worksheet->getCell('BI' . $rowIndex)->getValue() == 'Yes'){
                        HealthDetail::create([
                            'regID' => $applicant->regID,
                            'health_type' => 'Chronic Fatigue, Pain Or Seizure',
                            'addedBy' => auth()->user()->id,
                            'status'=>'New'
                        ]);
                    }

                    if($worksheet->getCell('BJ' . $rowIndex)->getValue() !== 'NULL' && str_contains($worksheet->getCell('BJ' . $rowIndex)->getValue(), 'Hearing')){
                        HealthDetail::create([
                            'regID' => $applicant->regID,
                            'health_type' => 'Hearing',
                            'addedBy' => auth()->user()->id,
                            'status'=>'New'
                        ]);
                    }

                    if($worksheet->getCell('BJ' . $rowIndex)->getValue() !== 'NULL' && str_contains($worksheet->getCell('BJ' . $rowIndex)->getValue(), 'Speech')){

                        HealthDetail::create([
                            'regID' => $applicant->regID,
                            'health_type' => 'Speech',
                            'addedBy' => auth()->user()->id,
                            'status'=>'New'
                        ]);

                    }

                }
                /*
                if($worksheet->getCell('G' . $rowIndex)->getValue() !== 'NULL' && $worksheet->getCell('G' . $rowIndex)->getValue() !== 'None' && $worksheet->getCell('G' . $rowIndex)->getValue() !== 'No'){

                    $village = Village::where('name', ucwords(strtolower($worksheet->getCell('G' . $rowIndex)->getValue())))->first();

                    if($village){
                        $applicant->village_id = $village->id;
                    }
                }*/

            }
            

            // Move to the next row
            $rowIndex++;
        }

        if(count($omitted_rows)>0){
            return back()->with('warning', 'New Beneficiary Records Added. The following rows in the spreadsheet were omitted:'.implode(', ',$omitted_rows));
        }else{
            return back()->with('success', 'New Beneficiary Records Added.');
        }

        
        
        
        
    }



    function generateRefID(Request $request)
    {
        $base_code = $this->getBaseRegID($request);
        // $existing_applicants = Applicant::where('regID','like',"$base_code%")->count();
        $existing_applicants = Applicant::orderBy('created_at','desc')->pluck('id')->first();
        $base_code.=sprintf("%03d", $existing_applicants+1);
        return $base_code;
    }

    function getBaseRegID(Request $request)
    {
        $prefix = 'MB';
        $dob = date('Ymd',strtotime($request->dobs));
        $gender = ($request->genders == 'Male')?1:2;

        $refID = "$prefix-".$gender;
        return $refID;
    }
    function normalizeString($string)
    {
        // Convert to lowercase
        $normalizedString = strtolower($string);

        // Replace slashes and dashes with a space
        $normalizedString = str_replace(['/', '-'], ' ', $normalizedString);

        // Remove special characters (if any) and extra spaces
        $normalizedString = preg_replace('/[^a-z0-9\s]/', '', $normalizedString);
        $normalizedString = preg_replace('/\s+/', ' ', $normalizedString);

        return trim($normalizedString);
    }

    function get_region_id($region_name){
        $region = Region::where('name', $region_name)->first();
        return $region->id ?? 0;
    }
    function get_county_id($county_name){
        $county = County::where('county', $county_name)->first();
        return $county->id ?? 0;
    }



}
