<?php

namespace App\Models;

use App\Models\Traits\NotifiedTrait;
use Illuminate\Contracts\Auth\MustVerifyEmail;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Relations\BelongsToMany;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Database\Eloquent\Relations\MorphMany;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;
use Laravel\Sanctum\HasApiTokens;
use Laratrust\Traits\LaratrustUserTrait;
use OwenIt\Auditing\Contracts\Auditable;

class User extends Authenticatable implements Auditable, MustVerifyEmail
{
    use LaratrustUserTrait;
    use HasApiTokens, HasFactory, Notifiable;
    use \OwenIt\Auditing\Auditable;
    use NotifiedTrait;
    use SoftDeletes;

    protected $guarded = [];

    /**
     * The attributes that should be hidden for serialization.
     *
     * @var array
     */
    protected $hidden = [
        'password',
        'remember_token',
    ];

    /**
     * The attributes that should be cast.
     *
     * @var array
     */
    protected $casts = [
        'email_verified_at' => 'datetime',
        'touring_areas' => 'array',
    ];

    /**
     * Get all application processes.
     */
    public function application_processes():MorphMany
    {
        return $this->morphMany(ApplicationsProcessing::class, 'applicable');
    }

    /**
     * Get all application processes.
     */
    public function generalNotifications()
    {
        if ($this->applicant()) {
            return GeneralNotification::whereIn('instance_id',$this->deployments()->pluck('id')->toArray())
                ->whereJsonContains('receivers',$this->id);
        }
        return null;
    }

    public function generalReadNotifications()
    {
        if ($this->applicant()) {
        return GeneralNotification::whereIn('instance_id',$this->deployments()->pluck('id')->toArray())
            ->whereJsonContains('has_read',$this->id);
        }
        return null;
    }

    /**
     * Get all application processes.
     */
    public function processed_applications():MorphMany
    {
        return $this->morphMany(Application::class, 'applicationable');
    }

    public function profilePicture($placeholder = 'assets/images/portrait/small/avatar-s-1.png')
    {
        if (!empty($this->profile_photo_path)) {
            return Storage::disk('public')->exists(config('filesystems.disks.profile_pictures.folder'). $this->profile_photo_path) ?
                Storage::disk('public')->url(config('filesystems.disks.profile_pictures.public_route') . $this->profile_photo_path) : asset($placeholder);
        }
        else return $placeholder;
    }

    public function roles()
    {
        return $this->belongsToMany(Role::class)
            ->wherePivotIn('instance_id', [getActiveInstance()]);
    }

    public function hasRoleById($id)
    {
        return ($this->roles()->where('id','=',$id)->count() > 0);
    }

    public function hasRole($name)
    {
        return ($this->roles()->where('name','=',$name)->count() > 0);
    }

    public function accessLog() {
        return $this->hasMany(AccessLog::class,'user_id');
    }

    public function invitations() {
        return $this->hasMany(DeploymentInvitation::class,'email','email');
    }

    public function appPhotos() {
        return $this->hasMany(AppPhoto::class,'user_id');
    }

    public function getActiveProfileAttribute() {
        $image = $this->appPhotos->where('status','=',1)->pluck('url')->first();

        if (isset($image)) return $image;
        else return 'assets/img/avatar.png';
    }

    public function profile()
    {
        return $this->hasOne(UserProfile::class, 'user_id');
    }

    public function office()
    {
        return $this->belongsTo(Office::class, 'office_id');
    }

    public function _gender()
    {
        return $this->belongsTo(Lookup::class, 'gender','id');
    }

    public function scopeActivated($query)
    {
        return $query->where('email_verified_at','!=',null);
    }

    public function scopeVerified($query)
    {
        return $query->where('account_verified_at','!=',null);
    }

    public function scopeInactive($query)
    {
        return $query->where('email_verified_at','=',null);
    }

    /**
     * Get a full name combination of first name, other name and last name
     *
     * @return string
     */
    public function getFullNameAttribute()
    {
        return "{$this->name}, {$this->firstname} {$this->othername}";
    }

    public function getAvatarUrlAttribute(){

    }

    /**
     * Get all applications.
     */
    public function applicationsAdded()
    {
        return $this->morphMany(Application::class, 'applicationable');
    }

    public function applications() {
        return $this->hasMany(Application::class,'applicant_number','applicant_number');
    }

    public function applicant() {
        return $this->hasOne(Applicant::class,'regID','applicant_number');
    }

    public function documents() {
        return $this->hasManyThrough(ApplicantDocument::class,Applicant::class,'regID','applicant_number');
    }

    public function announcements() {
        return $this->hasManyThrough(Announcement::class,AnnouncementsRead::class,'id','announcement_id');
    }

    public function customers($query) {
        return $query->where('is_admin','=',0);
    }
    public function getUnreadAnnouncementsAttribute() {
        $count = Announcement::published()->count() - auth()->user()->readAnnouncements->count();
        if ($count < 0) $count = 0;
        return $count;
    }

    public function readAnnouncements() {
        return $this->hasMany(AnnouncementsRead::class);
    }

    public function entries() {
        return $this->hasMany(Entry::class,'participant_id');
    }

    public function pendingSurveys()
    {

    }

    public function hasOrganisation()
    {
        return $this->hasOne(Client::class, 'admin')->first();
    }

    public function onboardingStage() {
        return $this->hasOne(OnboardingStage::class);
    }

    public function hasPendingInvitations()
    {
        return !(($this->invitations()->where('performed_activity', '=', false)->count() <= 0));
    }

    public function pendingInvitations()
    {
        return $this->invitations()->where('performed_activity', '=', false)->count();
    }

    public function deployments()
    {
        if (session()->get('role') == 'beneficiary') {
            return $this->applicant?->deployments();
        }
        return $this->belongsToMany(Deployment::class,);
    }

    public function client_()
    {
        return $this->belongsTo(Client::class,'client_id');
    }
    public function clients():BelongsToMany
    {
        return $this->belongsToMany(Client::class);
    }

    public function isChannelPartner():bool
    {
        if ($this->client_) {
            if (Str::slug($this->client_->field) == 'channel_partner' || $this->client_->field == 'Channel Partner') {
                return true;
            }
        }
        return false;
    }

    public function hasActiveSubscription():bool
    {
        if ($this->isChannelPartner()){
            if ($this->client_->subscriptions()->where('ends_at', '>', now())->orderByDesc('created_at')->count() > 0){
                return true;
            }
            else {
                return false;
            }
        }

        return true;
    }

    public function getActiveSubscriptionPlan()
    {
        if ($this->isChannelPartner()){
            return $this->client_?->subscriptions()->where('ends_at', '>', now())->orderByDesc('created_at')->pluck('plan_slug')->first();
        }
        return [];
    }

    public function isNew()
    {
        return !($this->clients()->count() > 0);
    }

    public function adminTo()
    {
        return $this->hasOne(Client::class, 'admin');
    }
    public function is_owner() {
        if (auth()->user()->hasRole('owner')) {
            return true;
        }
        return false;
    }

    public function children()
    {
        return $this->hasMany(User::class, 'supervisor')
            ->whereHas('roles',function ($query){
                $query->where('name','!=','beneficiary');
            });
    }
}
